# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gaphor',
 'gaphor.C4Model',
 'gaphor.C4Model.diagramitems',
 'gaphor.RAAML',
 'gaphor.RAAML.diagram_svgs',
 'gaphor.RAAML.fta',
 'gaphor.RAAML.stpa',
 'gaphor.SysML',
 'gaphor.SysML.blocks',
 'gaphor.SysML.requirements',
 'gaphor.UML',
 'gaphor.UML.actions',
 'gaphor.UML.classes',
 'gaphor.UML.deployments',
 'gaphor.UML.interactions',
 'gaphor.UML.profiles',
 'gaphor.UML.states',
 'gaphor.UML.usecases',
 'gaphor.codegen',
 'gaphor.core',
 'gaphor.core.modeling',
 'gaphor.core.styling',
 'gaphor.diagram',
 'gaphor.diagram.general',
 'gaphor.diagram.tools',
 'gaphor.extensions',
 'gaphor.plugins',
 'gaphor.plugins.autolayout',
 'gaphor.plugins.console',
 'gaphor.plugins.diagramexport',
 'gaphor.plugins.errorreports',
 'gaphor.plugins.xmiexport',
 'gaphor.services',
 'gaphor.storage',
 'gaphor.ui',
 'gaphor.ui.help']

package_data = \
{'': ['*'],
 'gaphor': ['locale/*',
            'locale/bn/LC_MESSAGES/*',
            'locale/ca/LC_MESSAGES/*',
            'locale/cs/LC_MESSAGES/*',
            'locale/de/LC_MESSAGES/*',
            'locale/es/LC_MESSAGES/*',
            'locale/fi/LC_MESSAGES/*',
            'locale/fr/LC_MESSAGES/*',
            'locale/gl/LC_MESSAGES/*',
            'locale/hr/LC_MESSAGES/*',
            'locale/hu/LC_MESSAGES/*',
            'locale/it/LC_MESSAGES/*',
            'locale/ja/LC_MESSAGES/*',
            'locale/ko/LC_MESSAGES/*',
            'locale/nl/LC_MESSAGES/*',
            'locale/pt_BR/LC_MESSAGES/*',
            'locale/ru/LC_MESSAGES/*',
            'locale/sl/LC_MESSAGES/*',
            'locale/sv/LC_MESSAGES/*',
            'locale/zh_Hans/LC_MESSAGES/*',
            'templates/*'],
 'gaphor.ui': ['icons/*',
               'icons/hicolor/scalable/actions/*',
               'icons/hicolor/scalable/apps/*',
               'icons/hicolor/scalable/emblems/*']}

install_requires = \
['PyGObject>=3.30,<4.0',
 'better-exceptions>=0.3.3,<0.4.0',
 'darkdetect>=0.5.1,<0.8.0',
 'gaphas>=3.1.0,<4.0.0',
 'generic>=1.0.0,<2.0.0',
 'jedi>=0.18.1,<0.19.0',
 'pycairo>=1.18,<2.0',
 'pydot>=1.4.2,<2.0.0',
 'tinycss2>=1.0.2,<2.0.0']

extras_require = \
{'docs': ['sphinx>=4.3,<5.4',
          'sphinx-rtd-theme>=1.0,<2.0',
          'sphinx-copybutton>=0.5.0,<0.6.0',
          'myst-nb>=0.17.1,<0.18.0']}

entry_points = \
{'babel.extractors': ['gaphor = gaphor.babel:extract_gaphor'],
 'console_scripts': ['gaphor = gaphor.ui:main',
                     'gaphorconvert = '
                     'gaphor.plugins.diagramexport.gaphorconvert:main'],
 'gaphor.appservices': ['app_file_manager = '
                        'gaphor.ui.appfilemanager:AppFileManager',
                        'error_reports = '
                        'gaphor.plugins.errorreports:ErrorReports',
                        'event_manager = gaphor.core.eventmanager:EventManager',
                        'greeter = gaphor.ui.greeter:Greeter',
                        'module_loader = '
                        'gaphor.services.moduleloader:ModuleLoader',
                        'self_test = gaphor.ui.selftest:SelfTest',
                        'styling = gaphor.ui.styling:Styling'],
 'gaphor.modelinglanguages': ['C4Model = '
                              'gaphor.C4Model.modelinglanguage:C4ModelLanguage',
                              'Core = '
                              'gaphor.core.modeling.modelinglanguage:CoreModelingLanguage',
                              'RAAML = '
                              'gaphor.RAAML.modelinglanguage:RAAMLModelingLanguage',
                              'SysML = '
                              'gaphor.SysML.modelinglanguage:SysMLModelingLanguage',
                              'UML = '
                              'gaphor.UML.modelinglanguage:UMLModelingLanguage'],
 'gaphor.modules': ['c4model_property_pages = '
                    'gaphor.C4Model.propertypages:init',
                    'general_ui_components = '
                    'gaphor.diagram.general.uicomponents:init',
                    'sysml_property_pages = gaphor.SysML.propertypages:init',
                    'uml_ui_components = gaphor.UML.uicomponents:init'],
 'gaphor.services': ['auto_layout = '
                     'gaphor.plugins.autolayout:AutoLayoutService',
                     'component_registry = '
                     'gaphor.services.componentregistry:ComponentRegistry',
                     'consolewindow = '
                     'gaphor.plugins.console.consolewindow:ConsoleWindow',
                     'copy = gaphor.services.copyservice:CopyService',
                     'diagram_export = '
                     'gaphor.plugins.diagramexport:DiagramExport',
                     'diagrams = gaphor.ui.diagrams:Diagrams',
                     'element_dispatcher = '
                     'gaphor.core.modeling.elementdispatcher:ElementDispatcher',
                     'element_factory = gaphor.core.modeling:ElementFactory',
                     'elementeditor = gaphor.ui.elementeditor:ElementEditor',
                     'event_manager = gaphor.core.eventmanager:EventManager',
                     'export_menu = gaphor.ui.menufragment:MenuFragment',
                     'file_manager = gaphor.ui.filemanager:FileManager',
                     'help = gaphor.ui.help:HelpService',
                     'main_window = gaphor.ui.mainwindow:MainWindow',
                     'modeling_language = '
                     'gaphor.services.modelinglanguage:ModelingLanguageService',
                     'properties = gaphor.services.properties:Properties',
                     'recent_files = gaphor.ui.recentfiles:RecentFiles',
                     'sanitizer = gaphor.UML.sanitizerservice:SanitizerService',
                     'toolbox = gaphor.ui.toolbox:Toolbox',
                     'tools_menu = gaphor.ui.menufragment:MenuFragment',
                     'tree_component = gaphor.ui.tree:TreeComponent',
                     'undo_manager = gaphor.services.undomanager:UndoManager',
                     'xmi_export = gaphor.plugins.xmiexport:XMIExport']}

setup_kwargs = {
    'name': 'gaphor',
    'version': '2.13.0',
    'description': 'Gaphor is the simple modeling tool written in Python.',
    'long_description': '<h1 align="center"><img src="https://github.com/gaphor/gaphor/blob/main/logos/gaphor-logo-full.svg?raw=true" alt="Gaphor - UML/SysML Modeling" height="96"/></h1>\n\n[![Build](https://github.com/gaphor/gaphor/actions/workflows/full-build.yml/badge.svg)](https://github.com/gaphor/gaphor/actions/workflows/full-build.yml?query=branch%3Amain)\n[![Hypothesis Tests](https://github.com/gaphor/gaphor/actions/workflows/hypothesis-test.yml/badge.svg)](https://github.com/gaphor/gaphor/actions/workflows/hypothesis-test.yml?query=branch%3Amain)\n[![Docs build state](https://readthedocs.org/projects/gaphor/badge/?version=latest)](https://gaphor.readthedocs.io)\n[![PyPI](https://img.shields.io/pypi/v/gaphor.svg)](https://pypi.org/project/gaphor)\n[![PyPI - Downloads](https://img.shields.io/pypi/dm/gaphor)](https://pypistats.org/packages/gaphor)\n[![Matrix](https://img.shields.io/badge/chat-on%20Matrix-success)](https://app.element.io/#/room/#gaphor_Lobby:gitter.im)\n\n[![Maintainability](https://api.codeclimate.com/v1/badges/f00974f5d7fe69fe4ecd/maintainability)](https://codeclimate.com/github/gaphor/gaphor/maintainability)\n[![Test Coverage](https://api.codeclimate.com/v1/badges/f00974f5d7fe69fe4ecd/test_coverage)](https://codeclimate.com/github/gaphor/gaphor/test_coverage)\n[![Translation Status](https://hosted.weblate.org/widgets/gaphor/-/gaphor/svg-badge.svg)](https://hosted.weblate.org/engage/gaphor)\n[![Sourcery](https://img.shields.io/badge/Sourcery-enabled-brightgreen)](https://sourcery.ai)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n[![standard-readme compliant](https://img.shields.io/badge/readme%20style-standard-brightgreen.svg)](https://github.com/RichardLitt/standard-readme)\n[![All Contributors](https://img.shields.io/badge/all_contributors-32-brightgreen.svg)](#contributors)\n\n\nGaphor is a UML and SysML modeling application written in Python.\nIt is designed to be easy to use, while still being powerful. Gaphor implements a fully-compliant UML 2 data model, so it is much more than a picture drawing tool. You can use Gaphor to quickly visualize different aspects of a system as well as create complete, highly complex models.\n\n<img alt="Gaphor Demo" src="https://raw.githubusercontent.com/gaphor/gaphor/main/docs/images/gaphor-demo.gif" style="display: block; margin: 2em auto">\n\n## 📑 Table of Contents\n\n- [Background](#background)\n- [Install](#floppy_disk-install)\n- [Usage](#usage)\n- [Contributing](#contributing)\n- [License](#license)\n\n## 📜 Background\n\nGaphor is a UML and SysML modeling application written in Python. We designed\nit to be easy to use, while still being powerful. Gaphor implements a\nfully-compliant UML 2 data model, so it is much more than a picture drawing\ntool. You can use Gaphor to quickly visualize different aspects of a system as\nwell as create complete, highly complex models.\n\nGaphor is designed around the following principles:\n\n- Simplicity: The application should be easy to use. Only some basic knowledge of UML or SysML is required.\n- Consistency: UML is a graphical modeling language, so all modeling is done in a diagram.\n- Workability: The application should not bother the user every time they do something non-UML-ish.\n\nGaphor is a GUI application that is built on [GTK](https://gtk.org) and\n[Cairo](https://www.cairographics.org/).\n[PyGObject](https://pygobject.readthedocs.io/) and\n[PyCairo](https://pycairo.readthedocs.io/) provide Python bindings for those\nlibraries. [Gaphas](https://github.com/gaphor/gaphas) provides the foundational\ndiagramming functionality.\n\n## 💾 Install\n\nYou can find [the latest version](https://gaphor.org/download) on the\n[gaphor.org website](https://gaphor.org/download). Gaphor ships installers for\nmacOS and Windows. Those can be found there. The Python package is also\n[available on PyPI](https://pypi.org/project/gaphor/).\n\nAll releases are available on\n[GitHub](https://github.com/gaphor/gaphor/releases/).\n\nIf you want to start developing on Gaphor, have a look at the [Installation\nsection of our Tech docs](https://gaphor.readthedocs.io/en/latest/).\n\n## 🔦 Usage\n### Creating models\n\nIf using Gaphor for the first time you will be presented with a greeter dialog\nat startup in which you can select one of 5 models available to you to work in:\n- **Generic:** (or blank) template\n- **UML:** *Unified Modeling Language* template\n- **SysML:** *Systems Modeling Language* template\n- **RAAML:** *Risk Analysis and Assessment Modeling language* template\n- **C4 Model:** *a lean graphical notation technique for modelling the architecture of software systems* template\n\nAfter you select a template, the main Gaphor Window will load, and you will be\nready to start modeling. Gaphor will automatically select the correct profile\nbased on the template that you selected, but you can also select other modeling\nprofiles if needed by clicking on the button next to the Profile dropdown menu\nat the top of your window.\n\nTo select an element you want to place, for example a Class, click on the icon\nin the Toolbox and then again on the diagram. This will place a new Class item\non the diagram and add a new Class to the model (it shows up in the Navigation).\nThe selected tool will reset itself to the Pointer tool if the option \'\'Diagram\n-> Reset tool\'\' is selected.\n\nPortions of the toolbox may also be collapsed depending on the type of diagram\nyou are modeling with. You can expand the collapsed portions of the toolbox if\nneeded.\n\n### Create a New Diagram\n\n1. Use the Navigation to select an element that can contain a diagram (a\nPackage or Profile)\n2. Select the New diagram button in the upper left and select the type of\ndiagram you would like to create from the dropdown.\n\nIt is also possible to create a new diagram by right clicking on a Package or\nProfile in the Navigation and selecting New Diagram.\n\n### Copy and Paste\n\nItems in a diagram can be copied and pasted in the same diagram or other\ndiagrams.\n\nCtrl+V: The default way to paste uses a link defining elements mode. This places\nan existing item in the diagram, but the item itself is not duplicated. In other\nwords, if you paste a Class object in a diagram, the Class will be added to the\ndiagram, but there will be no new Class in the Navigation.\n\nShift+Ctrl+V: Copy defining elements. This places a new item in the model by\nfully duplicating the item that was copied.\n\n### Drag and Drop\n\nAdding an existing element to a diagram is done by dragging the element from\nthe Navigation section onto a diagram. Diagrams and attribute/operations of a\nClass show up in the Navigation but can not be added to a diagram.\n\nElements can also be dragged within the Navigation in order to rearrange them\nin to different packages.\n\n## ♥ Contributing\n\nThanks goes to these wonderful people ([emoji key](https://github.com/kentcdodds/all-contributors#emoji-key)):\n\n<!-- ALL-CONTRIBUTORS-LIST:START - Do not remove or modify this section -->\n<!-- prettier-ignore-start -->\n<!-- markdownlint-disable -->\n<table>\n  <tbody>\n    <tr>\n      <td align="center"><a href="https://github.com/abjurstrom-torch"><img src="https://avatars1.githubusercontent.com/u/62608984?v=4?s=100" width="100px;" alt="Adam Bjurstrom"/><br /><sub><b>Adam Bjurstrom</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Aabjurstrom-torch" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="http://www.boduch.ca"><img src="https://avatars2.githubusercontent.com/u/114619?v=4?s=100" width="100px;" alt="Adam Boduch"/><br /><sub><b>Adam Boduch</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=adamboduch" title="Code">💻</a> <a href="https://github.com/gaphor/gaphor/commits?author=adamboduch" title="Tests">⚠️</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Aadamboduch" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/Alexander-Wilms"><img src="https://avatars2.githubusercontent.com/u/3226457?v=4?s=100" width="100px;" alt="Alexander Wilms"/><br /><sub><b>Alexander Wilms</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AAlexander-Wilms" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="http://www.aejh.co.uk"><img src="https://avatars1.githubusercontent.com/u/927233?v=4?s=100" width="100px;" alt="Alexis Howells"/><br /><sub><b>Alexis Howells</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=aejh" title="Documentation">📖</a></td>\n      <td align="center"><a href="https://ayanamy.is-a.dev/"><img src="https://avatars.githubusercontent.com/u/50583248?v=4?s=100" width="100px;" alt="Amy Y"/><br /><sub><b>Amy Y</b></sub></a><br /><a href="#translation-jy1263" title="Translation">🌍</a></td>\n      <td align="center"><a href="https://anreji.de"><img src="https://avatars.githubusercontent.com/u/69592284?v=4?s=100" width="100px;" alt="Andrew"/><br /><sub><b>Andrew</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Aanreji" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/amolenaar"><img src="https://avatars0.githubusercontent.com/u/96249?v=4?s=100" width="100px;" alt="Arjan Molenaar"/><br /><sub><b>Arjan Molenaar</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=amolenaar" title="Code">💻</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Aamolenaar" title="Bug reports">🐛</a> <a href="https://github.com/gaphor/gaphor/commits?author=amolenaar" title="Documentation">📖</a> <a href="https://github.com/gaphor/gaphor/pulls?q=is%3Apr+reviewed-by%3Aamolenaar" title="Reviewed Pull Requests">👀</a> <a href="#question-amolenaar" title="Answering Questions">💬</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Aamolenaar" title="Bug reports">🐛</a> <a href="#plugin-amolenaar" title="Plugin/utility libraries">🔌</a> <a href="https://github.com/gaphor/gaphor/commits?author=amolenaar" title="Tests">⚠️</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/Lutra-Fs"><img src="https://avatars.githubusercontent.com/u/36790218?v=4?s=100" width="100px;" alt="Bill ZHANG"/><br /><sub><b>Bill ZHANG</b></sub></a><br /><a href="#translation-Lutra-Fs" title="Translation">🌍</a></td>\n      <td align="center"><a href="https://github.com/blippost"><img src="https://avatars.githubusercontent.com/u/74373678?v=4?s=100" width="100px;" alt="Blippost"/><br /><sub><b>Blippost</b></sub></a><br /><a href="#ideas-Blippost" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="http://btibert3.github.io"><img src="https://avatars2.githubusercontent.com/u/203343?v=4?s=100" width="100px;" alt="Brock Tibert"/><br /><sub><b>Brock Tibert</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ABtibert3" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/can-lehmann"><img src="https://avatars.githubusercontent.com/u/85876381?v=4?s=100" width="100px;" alt="Can Lehmann"/><br /><sub><b>Can Lehmann</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=can-lehmann" title="Code">💻</a></td>\n      <td align="center"><a href="https://github.com/choff"><img src="https://avatars1.githubusercontent.com/u/309979?v=4?s=100" width="100px;" alt="Christian Hoff"/><br /><sub><b>Christian Hoff</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=choff" title="Code">💻</a></td>\n      <td align="center"><a href="https://github.com/DKX47"><img src="https://avatars.githubusercontent.com/u/12964384?v=4?s=100" width="100px;" alt="DKX47"/><br /><sub><b>DKX47</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ADKX47" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://ghuser.io/danyeaw"><img src="https://avatars1.githubusercontent.com/u/10014976?v=4?s=100" width="100px;" alt="Dan Yeaw"/><br /><sub><b>Dan Yeaw</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=danyeaw" title="Code">💻</a> <a href="https://github.com/gaphor/gaphor/commits?author=danyeaw" title="Tests">⚠️</a> <a href="https://github.com/gaphor/gaphor/commits?author=danyeaw" title="Documentation">📖</a> <a href="#platform-danyeaw" title="Packaging/porting to new platform">📦</a> <a href="#infra-danyeaw" title="Infrastructure (Hosting, Build-Tools, etc)">🚇</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Adanyeaw" title="Bug reports">🐛</a> <a href="#question-danyeaw" title="Answering Questions">💬</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/dk-zero-cool"><img src="https://avatars.githubusercontent.com/u/1395583?v=4?s=100" width="100px;" alt="Daniel Bergløv"/><br /><sub><b>Daniel Bergløv</b></sub></a><br /><a href="#ideas-dk-zero-cool" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="http://www.DanielNylander.se"><img src="https://avatars.githubusercontent.com/u/1206564?v=4?s=100" width="100px;" alt="Daniel Nylander"/><br /><sub><b>Daniel Nylander</b></sub></a><br /><a href="#translation-yeager" title="Translation">🌍</a></td>\n      <td align="center"><a href="https://github.com/DimShadoWWW"><img src="https://avatars2.githubusercontent.com/u/25516?v=4?s=100" width="100px;" alt="DimShadoWWW"/><br /><sub><b>DimShadoWWW</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ADimShadoWWW" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/Texopolis"><img src="https://avatars.githubusercontent.com/u/88953534?v=4?s=100" width="100px;" alt="Douglas B"/><br /><sub><b>Douglas B</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=Texopolis" title="Documentation">📖</a></td>\n      <td align="center"><a href="http://encolpe.wordpress.com"><img src="https://avatars1.githubusercontent.com/u/124361?v=4?s=100" width="100px;" alt="Encolpe DEGOUTE"/><br /><sub><b>Encolpe DEGOUTE</b></sub></a><br /><a href="#translation-encolpe" title="Translation">🌍</a></td>\n      <td align="center"><a href="https://github.com/egroeper"><img src="https://avatars3.githubusercontent.com/u/535113?v=4?s=100" width="100px;" alt="Enno Gröper"/><br /><sub><b>Enno Gröper</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=egroeper" title="Code">💻</a></td>\n      <td align="center"><a href="https://github.com/ezickler"><img src="https://avatars3.githubusercontent.com/u/3604310?v=4?s=100" width="100px;" alt="Enno Zickler"/><br /><sub><b>Enno Zickler</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Aezickler" title="Bug reports">🐛</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://bousse-e.univ-nantes.io"><img src="https://avatars.githubusercontent.com/u/5868014?v=4?s=100" width="100px;" alt="Erwan Bousse"/><br /><sub><b>Erwan Bousse</b></sub></a><br /><a href="#ideas-ebousse" title="Ideas, Planning, & Feedback">🤔</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Aebousse" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="http://www.frandieguez.dev"><img src="https://avatars.githubusercontent.com/u/4584?v=4?s=100" width="100px;" alt="Fran Diéguez"/><br /><sub><b>Fran Diéguez</b></sub></a><br /><a href="#translation-frandieguez" title="Translation">🌍</a></td>\n      <td align="center"><a href="https://github.com/gbrlgian"><img src="https://avatars.githubusercontent.com/u/47647695?v=4?s=100" width="100px;" alt="Gabriel Gian"/><br /><sub><b>Gabriel Gian</b></sub></a><br /><a href="#translation-gbrlgian" title="Translation">🌍</a></td>\n      <td align="center"><a href="https://github.com/liferooter"><img src="https://avatars.githubusercontent.com/u/54807745?v=4?s=100" width="100px;" alt="Gleb Smirnov"/><br /><sub><b>Gleb Smirnov</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Aliferooter" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="http://gjstewart.net"><img src="https://avatars.githubusercontent.com/u/7083701?v=4?s=100" width="100px;" alt="Greg Stewart"/><br /><sub><b>Greg Stewart</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AGregJohnStewart" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/Gytree"><img src="https://avatars.githubusercontent.com/u/28499079?v=4?s=100" width="100px;" alt="Gytree"/><br /><sub><b>Gytree</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AGytree" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="http://www.gunibert.de"><img src="https://avatars.githubusercontent.com/u/373597?v=4?s=100" width="100px;" alt="Günther Wagner"/><br /><sub><b>Günther Wagner</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Agwutz" title="Bug reports">🐛</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://www.hamishmb.com"><img src="https://avatars.githubusercontent.com/u/16725441?v=4?s=100" width="100px;" alt="Hamish Mcintyre-Bhatty"/><br /><sub><b>Hamish Mcintyre-Bhatty</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Ahamishmb" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/HighKingofMelons"><img src="https://avatars.githubusercontent.com/u/27958428?v=4?s=100" width="100px;" alt="HighKingofMelons"/><br /><sub><b>HighKingofMelons</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AHighKingofMelons" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://bandism.net/"><img src="https://avatars.githubusercontent.com/u/22633385?v=4?s=100" width="100px;" alt="Ikko Ashimine"/><br /><sub><b>Ikko Ashimine</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=eltociear" title="Code">💻</a></td>\n      <td align="center"><a href="http://jcrabill.weebly.com/"><img src="https://avatars.githubusercontent.com/u/5885545?v=4?s=100" width="100px;" alt="Jacob Crabill"/><br /><sub><b>Jacob Crabill</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AJacobCrabill" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/jischebeck"><img src="https://avatars0.githubusercontent.com/u/3011242?v=4?s=100" width="100px;" alt="Jan"/><br /><sub><b>Jan</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Ajischebeck" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://pfeifle.tech"><img src="https://avatars2.githubusercontent.com/u/23027708?v=4?s=100" width="100px;" alt="JensPfeifle"/><br /><sub><b>JensPfeifle</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=JensPfeifle" title="Documentation">📖</a></td>\n      <td align="center"><a href="https://github.com/vanillajonathan"><img src="https://avatars.githubusercontent.com/u/10222521?v=4?s=100" width="100px;" alt="Jonathan"/><br /><sub><b>Jonathan</b></sub></a><br /><a href="#ideas-vanillajonathan" title="Ideas, Planning, & Feedback">🤔</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Avanillajonathan" title="Bug reports">🐛</a> <a href="#translation-vanillajonathan" title="Translation">🌍</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://twitter.com/yonkeltron"><img src="https://avatars.githubusercontent.com/u/59451?v=4?s=100" width="100px;" alt="Jonathan E. Magen"/><br /><sub><b>Jonathan E. Magen</b></sub></a><br /><a href="#ideas-yonkeltron" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="https://jonnathanriquelmo.github.io/"><img src="https://avatars.githubusercontent.com/u/13298966?v=4?s=100" width="100px;" alt="JonnathanRiquelmo"/><br /><sub><b>JonnathanRiquelmo</b></sub></a><br /><a href="#translation-JonnathanRiquelmo" title="Translation">🌍</a></td>\n      <td align="center"><a href="https://oskuro.net/"><img src="https://avatars3.githubusercontent.com/u/929712?v=4?s=100" width="100px;" alt="Jordi Mallach"/><br /><sub><b>Jordi Mallach</b></sub></a><br /><a href="#translation-jmallach" title="Translation">🌍</a></td>\n      <td align="center"><a href="https://github.com/dlagg"><img src="https://avatars3.githubusercontent.com/u/44321931?v=4?s=100" width="100px;" alt="Jorge DLG"/><br /><sub><b>Jorge DLG</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Adlagg" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/h8672"><img src="https://avatars.githubusercontent.com/u/8805540?v=4?s=100" width="100px;" alt="Juha-Matti Kokkonen"/><br /><sub><b>Juha-Matti Kokkonen</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Ah8672" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://jrueberg.de"><img src="https://avatars.githubusercontent.com/u/22551563?v=4?s=100" width="100px;" alt="Julius Rüberg"/><br /><sub><b>Julius Rüberg</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AToorero" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/JuliusBrueggemann"><img src="https://avatars.githubusercontent.com/u/35491022?v=4?s=100" width="100px;" alt="JuliusBrueggemann"/><br /><sub><b>JuliusBrueggemann</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AJuliusBrueggemann" title="Bug reports">🐛</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="http://twitter.com/kapilvt"><img src="https://avatars3.githubusercontent.com/u/21650?v=4?s=100" width="100px;" alt="Kapil Thangavelu"/><br /><sub><b>Kapil Thangavelu</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Akapilt" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/KhazAkar"><img src="https://avatars.githubusercontent.com/u/12693890?v=4?s=100" width="100px;" alt="KhazAkar"/><br /><sub><b>KhazAkar</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AKhazAkar" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://www.kianmeng.org"><img src="https://avatars.githubusercontent.com/u/134518?v=4?s=100" width="100px;" alt="Kian-Meng Ang"/><br /><sub><b>Kian-Meng Ang</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=kianmeng" title="Documentation">📖</a></td>\n      <td align="center"><a href="https://github.com/Lazerbeak12345"><img src="https://avatars.githubusercontent.com/u/22641188?v=4?s=100" width="100px;" alt="Lazerbeak12345"/><br /><sub><b>Lazerbeak12345</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ALazerbeak12345" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/LordSinSentido"><img src="https://avatars.githubusercontent.com/u/57022857?v=4?s=100" width="100px;" alt="Lordy"/><br /><sub><b>Lordy</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ALordSinSentido" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="http://monkington.github.io"><img src="https://avatars.githubusercontent.com/u/778856?v=4?s=100" width="100px;" alt="Mark Kennedy"/><br /><sub><b>Mark Kennedy</b></sub></a><br /><a href="#ideas-mrmonkington" title="Ideas, Planning, & Feedback">🤔</a> <a href="https://github.com/gaphor/gaphor/commits?author=mrmonkington" title="Code">💻</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Amrmonkington" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/markdluethje"><img src="https://avatars.githubusercontent.com/u/31922494?v=4?s=100" width="100px;" alt="Mark-Daniel Lüthje"/><br /><sub><b>Mark-Daniel Lüthje</b></sub></a><br /><a href="#translation-markdluethje" title="Translation">🌍</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/mathiascode"><img src="https://avatars.githubusercontent.com/u/8754153?v=4?s=100" width="100px;" alt="Mat"/><br /><sub><b>Mat</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=mathiascode" title="Code">💻</a></td>\n      <td align="center"><a href="https://github.com/kainne44"><img src="https://avatars.githubusercontent.com/u/50899654?v=4?s=100" width="100px;" alt="Matthew Maclaine"/><br /><sub><b>Matthew Maclaine</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Akainne44" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="http://bendiks.de"><img src="https://avatars.githubusercontent.com/u/8329544?v=4?s=100" width="100px;" alt="Maxim"/><br /><sub><b>Maxim</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Aascaron37" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/Mek101"><img src="https://avatars.githubusercontent.com/u/34246799?v=4?s=100" width="100px;" alt="Mek101"/><br /><sub><b>Mek101</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AMek101" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/c4tachan"><img src="https://avatars.githubusercontent.com/u/2130211?v=4?s=100" width="100px;" alt="Michael Patrick Tkacik"/><br /><sub><b>Michael Patrick Tkacik</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Ac4tachan" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/mbessonov"><img src="https://avatars2.githubusercontent.com/u/172974?v=4?s=100" width="100px;" alt="Mikhail Bessonov"/><br /><sub><b>Mikhail Bessonov</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Ambessonov" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="http://nedko.arnaudov.name"><img src="https://avatars2.githubusercontent.com/u/96399?v=4?s=100" width="100px;" alt="Nedko Arnaudov"/><br /><sub><b>Nedko Arnaudov</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Anedko" title="Bug reports">🐛</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/philwilkinson40"><img src="https://avatars.githubusercontent.com/u/27684871?v=4?s=100" width="100px;" alt="Phil_Smurf"/><br /><sub><b>Phil_Smurf</b></sub></a><br /><a href="#ideas-philwilkinson40" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="http://www.rmunoz.net"><img src="https://avatars2.githubusercontent.com/u/23944?v=4?s=100" width="100px;" alt="Rafael Muñoz Cárdenas"/><br /><sub><b>Rafael Muñoz Cárdenas</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AMenda" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/RenStone83"><img src="https://avatars.githubusercontent.com/u/106451678?v=4?s=100" width="100px;" alt="RenStone83"/><br /><sub><b>RenStone83</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ARenStone83" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/ruimaciel"><img src="https://avatars3.githubusercontent.com/u/169121?v=4?s=100" width="100px;" alt="Rui Maciel"/><br /><sub><b>Rui Maciel</b></sub></a><br /><a href="#ideas-ruimaciel" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="http://www.uni-kassel.de/go/holzhauer"><img src="https://avatars.githubusercontent.com/u/1692563?v=4?s=100" width="100px;" alt="Sascha Holzhauer"/><br /><sub><b>Sascha Holzhauer</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AHolzhauer" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/Seppli11"><img src="https://avatars.githubusercontent.com/u/9285506?v=4?s=100" width="100px;" alt="Sebi"/><br /><sub><b>Sebi</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ASeppli11" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/darkcircle"><img src="https://avatars.githubusercontent.com/u/1160498?v=4?s=100" width="100px;" alt="Seong-ho Cho"/><br /><sub><b>Seong-ho Cho</b></sub></a><br /><a href="#translation-darkcircle" title="Translation">🌍</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/artscoop"><img src="https://avatars.githubusercontent.com/u/1023091?v=4?s=100" width="100px;" alt="Steve Kossouho"/><br /><sub><b>Steve Kossouho</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Aartscoop" title="Bug reports">🐛</a> <a href="https://github.com/gaphor/gaphor/commits?author=artscoop" title="Code">💻</a></td>\n      <td align="center"><a href="http://stevenliu216.github.io"><img src="https://avatars3.githubusercontent.com/u/1274417?v=4?s=100" width="100px;" alt="Steven Liu"/><br /><sub><b>Steven Liu</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Astevenliu216" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="http://www.slide.nz"><img src="https://avatars.githubusercontent.com/u/47554072?v=4?s=100" width="100px;" alt="Thomas"/><br /><sub><b>Thomas</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ACoderThomasB" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/TiemenSch"><img src="https://avatars.githubusercontent.com/u/7141863?v=4?s=100" width="100px;" alt="Tiemen Schuijbroek"/><br /><sub><b>Tiemen Schuijbroek</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ATiemenSch" title="Bug reports">🐛</a> <a href="https://github.com/gaphor/gaphor/commits?author=TiemenSch" title="Code">💻</a></td>\n      <td align="center"><a href="http://tobiasbernard.com"><img src="https://avatars3.githubusercontent.com/u/1908896?v=4?s=100" width="100px;" alt="Tobias Bernard"/><br /><sub><b>Tobias Bernard</b></sub></a><br /><a href="#design-bertob" title="Design">🎨</a> <a href="#ideas-bertob" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="https://github.com/TomBous"><img src="https://avatars.githubusercontent.com/u/10131977?v=4?s=100" width="100px;" alt="TomBous"/><br /><sub><b>TomBous</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3ATomBous" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/tomaszdrozdz"><img src="https://avatars.githubusercontent.com/u/14263613?v=4?s=100" width="100px;" alt="Tomasz Drożdż"/><br /><sub><b>Tomasz Drożdż</b></sub></a><br /><a href="#ideas-tomaszdrozdz" title="Ideas, Planning, & Feedback">🤔</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/tonytheleg"><img src="https://avatars3.githubusercontent.com/u/43508092?v=4?s=100" width="100px;" alt="Tony"/><br /><sub><b>Tony</b></sub></a><br /><a href="#maintenance-tonytheleg" title="Maintenance">🚧</a></td>\n      <td align="center"><a href="https://github.com/Viicos"><img src="https://avatars.githubusercontent.com/u/65306057?v=4?s=100" width="100px;" alt="Viicos"/><br /><sub><b>Viicos</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3AViicos" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/Xander982"><img src="https://avatars2.githubusercontent.com/u/51178927?v=4?s=100" width="100px;" alt="Xander982"/><br /><sub><b>Xander982</b></sub></a><br /><a href="#content-Xander982" title="Content">🖋</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3AXander982" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/yantaozhao"><img src="https://avatars.githubusercontent.com/u/9587405?v=4?s=100" width="100px;" alt="YantaoZhao"/><br /><sub><b>YantaoZhao</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Ayantaozhao" title="Bug reports">🐛</a> <a href="#ideas-yantaozhao" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="https://github.com/actionless"><img src="https://avatars1.githubusercontent.com/u/1655669?v=4?s=100" width="100px;" alt="Yauhen Kirylau"/><br /><sub><b>Yauhen Kirylau</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=actionless" title="Documentation">📖</a> <a href="#platform-actionless" title="Packaging/porting to new platform">📦</a> <a href="#ideas-actionless" title="Ideas, Planning, & Feedback">🤔</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Aactionless" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/sz332"><img src="https://avatars.githubusercontent.com/u/8182138?v=4?s=100" width="100px;" alt="Zsolt Sandor"/><br /><sub><b>Zsolt Sandor</b></sub></a><br /><a href="#ideas-sz332" title="Ideas, Planning, & Feedback">🤔</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Asz332" title="Bug reports">🐛</a> <a href="https://github.com/gaphor/gaphor/commits?author=sz332" title="Code">💻</a> <a href="https://github.com/gaphor/gaphor/commits?author=sz332" title="Documentation">📖</a></td>\n      <td align="center"><a href="http://www.zorinos.com"><img src="https://avatars1.githubusercontent.com/u/34811668?v=4?s=100" width="100px;" alt="albanobattistella"/><br /><sub><b>albanobattistella</b></sub></a><br /><a href="#translation-albanobattistella" title="Translation">🌍</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/deifemu"><img src="https://avatars.githubusercontent.com/u/81991548?v=4?s=100" width="100px;" alt="deifemu"/><br /><sub><b>deifemu</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Adeifemu" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/freddii"><img src="https://avatars0.githubusercontent.com/u/7213207?v=4?s=100" width="100px;" alt="freddii"/><br /><sub><b>freddii</b></sub></a><br /><a href="#ideas-freddii" title="Ideas, Planning, & Feedback">🤔</a> <a href="https://github.com/gaphor/gaphor/commits?author=freddii" title="Documentation">📖</a></td>\n      <td align="center"><a href="https://github.com/freezed-or-frozen"><img src="https://avatars.githubusercontent.com/u/28558919?v=4?s=100" width="100px;" alt="freezed-or-frozen"/><br /><sub><b>freezed-or-frozen</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Afreezed-or-frozen" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/fu7mu4"><img src="https://avatars.githubusercontent.com/u/1885537?v=4?s=100" width="100px;" alt="fu7mu4"/><br /><sub><b>fu7mu4</b></sub></a><br /><a href="#translation-fu7mu4" title="Translation">🌍</a></td>\n      <td align="center"><a href="https://gavr123456789.gitlab.io/hugo-test/"><img src="https://avatars3.githubusercontent.com/u/30507409?v=4?s=100" width="100px;" alt="gavr123456789"/><br /><sub><b>gavr123456789</b></sub></a><br /><a href="#ideas-gavr123456789" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="https://github.com/greedyj4ck"><img src="https://avatars.githubusercontent.com/u/33233389?v=4?s=100" width="100px;" alt="greedyj4ck"/><br /><sub><b>greedyj4ck</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Agreedyj4ck" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/kellenmoura"><img src="https://avatars0.githubusercontent.com/u/69016459?v=4?s=100" width="100px;" alt="kellenmoura"/><br /><sub><b>kellenmoura</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Akellenmoura" title="Bug reports">🐛</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/lukman83"><img src="https://avatars.githubusercontent.com/u/69509634?v=4?s=100" width="100px;" alt="lukman83"/><br /><sub><b>lukman83</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Alukman83" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/melisdogan"><img src="https://avatars2.githubusercontent.com/u/33630433?v=4?s=100" width="100px;" alt="melisdogan"/><br /><sub><b>melisdogan</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=melisdogan" title="Documentation">📖</a></td>\n      <td align="center"><a href="https://github.com/milotype"><img src="https://avatars.githubusercontent.com/u/43657314?v=4?s=100" width="100px;" alt="milotype"/><br /><sub><b>milotype</b></sub></a><br /><a href="#translation-milotype" title="Translation">🌍</a> <a href="https://github.com/gaphor/gaphor/commits?author=milotype" title="Documentation">📖</a></td>\n      <td align="center"><a href="https://github.com/mskorkowski"><img src="https://avatars.githubusercontent.com/u/90662755?v=4?s=100" width="100px;" alt="mskorkowski"/><br /><sub><b>mskorkowski</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Amskorkowski" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/noblevirk"><img src="https://avatars.githubusercontent.com/u/5385824?v=4?s=100" width="100px;" alt="noblevirk"/><br /><sub><b>noblevirk</b></sub></a><br /><a href="#ideas-noblevirk" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="https://github.com/nomisge"><img src="https://avatars.githubusercontent.com/u/16142147?v=4?s=100" width="100px;" alt="nomisge"/><br /><sub><b>nomisge</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Anomisge" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/ovari"><img src="https://avatars.githubusercontent.com/u/17465872?v=4?s=100" width="100px;" alt="ovari"/><br /><sub><b>ovari</b></sub></a><br /><a href="#ideas-ovari" title="Ideas, Planning, & Feedback">🤔</a> <a href="#translation-ovari" title="Translation">🌍</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Aovari" title="Bug reports">🐛</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/perovsek"><img src="https://avatars.githubusercontent.com/u/115713273?v=4?s=100" width="100px;" alt="perovsek"/><br /><sub><b>perovsek</b></sub></a><br /><a href="#translation-perovsek" title="Translation">🌍</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Aperovsek" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/qsodev"><img src="https://avatars.githubusercontent.com/u/11365470?v=4?s=100" width="100px;" alt="qsodev"/><br /><sub><b>qsodev</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Aqsodev" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/samirodj"><img src="https://avatars.githubusercontent.com/u/36422980?v=4?s=100" width="100px;" alt="samirodj"/><br /><sub><b>samirodj</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Asamirodj" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/seryafarma"><img src="https://avatars0.githubusercontent.com/u/3274071?v=4?s=100" width="100px;" alt="seryafarma"/><br /><sub><b>seryafarma</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=seryafarma" title="Documentation">📖</a></td>\n      <td align="center"><a href="https://github.com/icfmp"><img src="https://avatars.githubusercontent.com/u/71736258?v=4?s=100" width="100px;" alt="sib@c"/><br /><sub><b>sib@c</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Aicfmp" title="Bug reports">🐛</a> <a href="#ideas-icfmp" title="Ideas, Planning, & Feedback">🤔</a></td>\n      <td align="center"><a href="https://github.com/tronta"><img src="https://avatars1.githubusercontent.com/u/5135577?v=4?s=100" width="100px;" alt="tronta"/><br /><sub><b>tronta</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/issues?q=author%3Atronta" title="Bug reports">🐛</a></td>\n      <td align="center"><a href="https://github.com/vikdevelop"><img src="https://avatars.githubusercontent.com/u/83600218?v=4?s=100" width="100px;" alt="vikdevelop"/><br /><sub><b>vikdevelop</b></sub></a><br /><a href="#translation-vikdevelop" title="Translation">🌍</a></td>\n    </tr>\n    <tr>\n      <td align="center"><a href="https://github.com/wrobell"><img src="https://avatars2.githubusercontent.com/u/105664?v=4?s=100" width="100px;" alt="wrobell"/><br /><sub><b>wrobell</b></sub></a><br /><a href="https://github.com/gaphor/gaphor/commits?author=wrobell" title="Code">💻</a> <a href="https://github.com/gaphor/gaphor/commits?author=wrobell" title="Tests">⚠️</a> <a href="https://github.com/gaphor/gaphor/issues?q=author%3Awrobell" title="Bug reports">🐛</a> <a href="#design-wrobell" title="Design">🎨</a></td>\n      <td align="center"><a href="https://github.com/oscfdezdz"><img src="https://avatars.githubusercontent.com/u/42654671?v=4?s=100" width="100px;" alt="Óscar Fernández Díaz"/><br /><sub><b>Óscar Fernández Díaz</b></sub></a><br /><a href="#translation-oscfdezdz" title="Translation">🌍</a> <a href="https://github.com/gaphor/gaphor/commits?author=oscfdezdz" title="Code">💻</a></td>\n      <td align="center"><a href="https://github.com/zlv"><img src="https://avatars.githubusercontent.com/u/602210?v=4?s=100" width="100px;" alt="Евгений Лежнин"/><br /><sub><b>Евгений Лежнин</b></sub></a><br /><a href="#translation-zlv" title="Translation">🌍</a></td>\n    </tr>\n  </tbody>\n</table>\n\n<!-- markdownlint-restore -->\n<!-- prettier-ignore-end -->\n\n<!-- ALL-CONTRIBUTORS-LIST:END -->\n\nThis project follows the\n[all-contributors](https://github.com/kentcdodds/all-contributors)\nspecification. Contributions of any kind are welcome!\n\n1.  Check for open issues or open a fresh issue to start a discussion\n    around a feature idea or a bug. There is a\n    [first-timers-only](https://github.com/gaphor/gaphor/issues?utf8=%E2%9C%93&q=is%3Aissue+is%3Aopen+label%3Afirst-timers-only)\n    tag for issues that should be ideal for people who are not very\n    familiar with the codebase yet.\n2.  Fork [the repository](https://github.com/gaphor/gaphor) on\n    GitHub to start making your changes to the **main** branch (or\n    branch off of it).\n3.  Write a test which shows that the bug was fixed or that the feature\n    works as expected.\n4.  Send a pull request and bug the maintainers until it gets merged and\n    published. :smile:\n\nSee [the contributing file](CONTRIBUTING.md)!\n\n### Translations\n\nTranslation of Gaphor is mostly done using [Weblate](https://hosted.weblate.org/projects/gaphor/gaphor/).\n\nFor the Linux Flatpak, the desktop entry comment string can be translated at our [Flatpak repository](https://github.com/flathub/org.gaphor.Gaphor/blob/master/share/org.gaphor.Gaphor.desktop).\n\nThank you so much for your effort in helping us keep it available in many languages!\n\n## © License\n\nCopyright © Arjan Molenaar and Dan Yeaw\n\nLicensed under the [Apache License v2](LICENSE.txt).\n\nSummary: You can do what you like with Gaphor, as long as you include the\nrequired notices. This permissive license contains a patent license from the\ncontributors of the code.\n',
    'author': 'Arjan J. Molenaar',
    'author_email': 'gaphor@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://gaphor.org/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<3.12',
}


setup(**setup_kwargs)



"""1:n and n:m relations in the data model are saved using a collection."""

from typing import Generic, List, Type, TypeVar, overload

from gaphor.core.modeling.event import AssociationUpdated
from gaphor.core.modeling.listmixins import querymixin, recursemixin, recurseproxy

T = TypeVar("T")


class collectionlist(recursemixin, querymixin, List[T]):  # type: ignore[misc]
    """
    >>> c = collectionlist()
    >>> c.append("a")
    >>> c.append("b")
    >>> c.append("c")
    >>> c
    ['a', 'b', 'c']

    It should work with the datamodel too:

    >>> from gaphor.UML import *
    >>> c = Class()
    >>> c.ownedOperation = Operation()
    >>> c.ownedOperation  # doctest: +ELLIPSIS
    [<gaphor.UML.uml.Operation element ...>]
    >>> c.ownedOperation[0]  # doctest: +ELLIPSIS
    <gaphor.UML.uml.Operation element ...>
    >>> c.ownedOperation = Operation()
    >>> c.ownedOperation[0].ownedParameter = Parameter()
    >>> c.ownedOperation[0].ownedParameter = Parameter()
    >>> c.ownedOperation[0].ownedParameter[0].name = "foo"
    >>> c.ownedOperation[0].ownedParameter[0].name
    'foo'
    >>> c.ownedOperation[0].ownedParameter[1].name = "bar"
    >>> list(c.ownedOperation[0].ownedParameter[:].name)
    ['foo', 'bar']
    >>> c.ownedOperation[:].ownedParameter.name  # doctest: +ELLIPSIS
    <gaphor.core.modeling.listmixins.recurseproxy object at 0x...>
    >>> list(c.ownedOperation[:].ownedParameter.name)
    ['foo', 'bar']
    >>> c.ownedOperation[0].ownedParameter['it.name=="foo"', 0].name
    'foo'
    >>> c.ownedOperation[:].ownedParameter['it.name=="foo"', 0].name
    'foo'
    """


class collection(Generic[T]):
    """Collection (set-like) for model elements' 1:n and n:m relationships."""

    def __init__(self, property, object, type: Type[T]):
        self.property = property
        self.object = object
        self.type = type
        self.items: collectionlist[T] = collectionlist()

    def __len__(self) -> int:
        return len(self.items)

    def __setitem__(self, key, value) -> None:
        raise RuntimeError("items should not be overwritten.")

    def __delitem__(self, key: T) -> None:
        self.remove(key)

    @overload
    def __getitem__(self, key: int) -> T:
        ...

    @overload  # Literal[slice(None, None, None)]
    def __getitem__(self, key: slice) -> recurseproxy[T]:
        ...

    def __getitem__(self, key):
        return self.items.__getitem__(key)

    def __contains__(self, obj) -> bool:
        return self.items.__contains__(obj)

    def __iter__(self):
        return iter(self.items)

    def __str__(self):
        return str(self.items)

    __repr__ = __str__

    def __bool__(self):
        return self.items != []

    def append(self, value: T) -> None:
        if isinstance(value, self.type):
            self.property.set(self.object, value)
        else:
            raise TypeError(f"Object is not of type {self.type.__name__}")

    def remove(self, value: T) -> None:
        if value in self.items:
            self.property.delete(self.object, value)

    def index(self, key: T) -> int:
        """Given an object, return the position of that object in the
        collection."""
        return self.items.index(key)

    # OCL members (from SMW by Ivan Porres, http://www.abo.fi/~iporres/smw)

    def size(self):
        return len(self.items)

    def includes(self, o):
        return o in self.items

    def excludes(self, o):
        return not self.includes(o)

    def count(self, o):
        return self.items.count(o)

    def includesAll(self, c):
        return next((0 for o in c if o not in self.items), 1)

    def excludesAll(self, c):
        return next((0 for o in c if o in self.items), 1)

    def select(self, f):
        return [v for v in self.items if f(v)]

    def reject(self, f):
        return [v for v in self.items if not f(v)]

    def collect(self, f):
        return [f(v) for v in self.items]

    def isEmpty(self):
        return len(self.items) == 0

    def nonEmpty(self):
        return not self.isEmpty()

    def swap(self, item1, item2):
        """Swap two elements.

        Return true if swap was successful.
        """
        try:
            i1 = self.items.index(item1)
            i2 = self.items.index(item2)
            self.items[i1], self.items[i2] = self.items[i2], self.items[i1]

            self.object.handle(AssociationUpdated(self.object, self.property))
            return True
        except (IndexError, ValueError):
            return False

    def order(self, key):
        self.items.sort(key=key)
        self.object.handle(AssociationUpdated(self.object, self.property))



from gaphor.diagram.drop import drop
from gaphor.diagram.presentation import connect
from gaphor.diagram.support import get_diagram_item
from gaphor.SysML.sysml import ProxyPort
from gaphor.UML.drop import diagram_has_presentation


@drop.register
def drop_proxy_port(element: ProxyPort, diagram, x, y):
    item_class = get_diagram_item(type(element))
    if not item_class:
        return None

    head_item = diagram_has_presentation(diagram, element.encapsulatedClassifier)
    if not head_item:
        return None

    item = diagram.create(item_class)
    assert item

    item.matrix.translate(x, y)
    item.subject = element

    connect(item, item.handles()[0], head_item)

    return item



from gaphor.UML.states import copypaste, group, vertexconnect
from gaphor.UML.states.finalstate import FinalStateItem
from gaphor.UML.states.pseudostates import PseudostateItem
from gaphor.UML.states.state import StateItem
from gaphor.UML.states.statemachine import StateMachineItem
from gaphor.UML.states.transition import TransitionItem

__all__ = [
    "FinalStateItem",
    "PseudostateItem",
    "StateItem",
    "StateMachineItem",
    "TransitionItem",
]



from gaphor.core.modeling.coremodel import Comment
from gaphor.core.modeling.diagram import Diagram, DrawContext, UpdateContext
from gaphor.core.modeling.element import Element, self_and_owners
from gaphor.core.modeling.elementfactory import ElementFactory
from gaphor.core.modeling.event import *
from gaphor.core.modeling.presentation import Presentation
from gaphor.core.modeling.stylesheet import StyleSheet



"""Definitions (types) for style sheets."""
from enum import Enum
from typing import Sequence, Tuple, TypedDict, Union

Color = Tuple[float, float, float, float]  # RGBA
Padding = Tuple[float, float, float, float]  # top/right/bottom/left
Number = Union[int, float]


class TextAlign(Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class VerticalAlign(Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class JustifyContent(Enum):
    CENTER = "center"
    END = "end"
    START = "start"
    STRETCH = "stretch"


class FontStyle(Enum):
    NORMAL = "normal"
    ITALIC = "italic"


class FontWeight(Enum):
    NORMAL = "normal"
    BOLD = "bold"


class TextDecoration(Enum):
    NONE = "none"
    UNDERLINE = "underline"


# Style is using SVG properties where possible
# https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute
Style = TypedDict(
    "Style",
    {
        "background-color": Color,
        "border-radius": Number,
        "color": Color,
        "dash-style": Sequence[Number],
        "padding": Padding,
        "font-family": str,
        "font-size": Union[int, float, str],
        "font-style": FontStyle,
        "font-weight": FontWeight,
        "justify-content": JustifyContent,
        "line-style": Number,
        "line-width": Number,
        "min-width": Number,
        "min-height": Number,
        "opacity": Number,
        "text-decoration": TextDecoration,
        "text-align": TextAlign,
        "text-color": Color,
        "vertical-align": VerticalAlign,
        "vertical-spacing": Number,
    },
    total=False,
)



import math

from gaphor import UML
from gaphor.core import transactional
from gaphor.diagram.propertypages import (
    PropertyPageBase,
    PropertyPages,
    handler_blocking,
    new_resource_builder,
)
from gaphor.UML.actions.activitynodes import DecisionNodeItem, ForkNodeItem
from gaphor.UML.actions.objectnode import ObjectNodeItem

new_builder = new_resource_builder("gaphor.UML.actions")


@PropertyPages.register(ObjectNodeItem)
class ObjectNodePropertyPage(PropertyPageBase):

    order = 15

    ORDERING_VALUES = ["unordered", "ordered", "LIFO", "FIFO"]

    subject: UML.ObjectNode
    item: ObjectNodeItem

    def __init__(self, item):
        self.item = item

    def construct(self):
        subject = self.item.subject

        if not subject:
            return

        builder = new_builder(
            "object-node-editor",
            signals={
                "upper-bound-changed": (self._on_upper_bound_change,),
                "ordering-changed": (self._on_ordering_change,),
                "show-ordering-changed": (self._on_ordering_show_change,),
            },
        )

        upper_bound = builder.get_object("upper-bound")
        upper_bound.set_text(subject.upperBound or "")

        ordering = builder.get_object("ordering")
        ordering.set_active(self.ORDERING_VALUES.index(subject.ordering))

        show_ordering = builder.get_object("show-ordering")
        show_ordering.set_active(self.item.show_ordering)

        return builder.get_object("object-node-editor")

    @transactional
    def _on_upper_bound_change(self, entry):
        value = entry.get_text().strip()
        self.item.subject.upperBound = value

    @transactional
    def _on_ordering_change(self, combo):
        value = self.ORDERING_VALUES[combo.get_active()]
        self.item.subject.ordering = value

    @transactional
    def _on_ordering_show_change(self, button, gparam):
        self.item.show_ordering = button.get_active()


@PropertyPages.register(DecisionNodeItem)
class DecisionNodePropertyPage(PropertyPageBase):

    order = 20

    def __init__(self, item):
        self.item = item

    def construct(self):
        builder = new_builder(
            "decision-node-editor",
            signals={"show-type-changed": (self._on_show_type_change,)},
        )

        show_type = builder.get_object("show-type")
        show_type.set_active(self.item.show_underlaying_type)

        return builder.get_object("decision-node-editor")

    @transactional
    def _on_show_type_change(self, button, gparam):
        self.item.show_underlaying_type = button.get_active()


@PropertyPages.register(ForkNodeItem)
class ForkNodePropertyPage(PropertyPageBase):

    order = 20

    def __init__(self, item):
        self.item = item

    def construct(self):
        builder = new_builder(
            "fork-node-editor",
            signals={"horizontal-changed": (self._on_horizontal_change,)},
        )

        horizontal = builder.get_object("horizontal")
        horizontal.set_active(self.is_horizontal())

        return builder.get_object("fork-node-editor")

    def is_horizontal(self):
        return self.item.matrix[2] != 0

    @transactional
    def _on_horizontal_change(self, button, gparam):
        active = button.get_active()
        horizontal = self.is_horizontal()
        if active and not horizontal:
            self.item.matrix.rotate(math.pi / 2)
        elif not active and horizontal:
            self.item.matrix.rotate(-math.pi / 2)
        self.item.request_update()


@PropertyPages.register(UML.JoinNode)
class JoinNodePropertyPage(PropertyPageBase):

    order = 15

    subject: UML.JoinNode

    def __init__(self, subject):
        self.subject = subject

    def construct(self):
        subject = self.subject

        if not subject:
            return

        builder = new_builder(
            "join-node-editor",
            signals={"join-spec-changed": (self._on_join_spec_change,)},
        )

        join_spec = builder.get_object("join-spec")
        join_spec.set_text(subject.joinSpec or "")

        return builder.get_object("join-node-editor")

    @transactional
    def _on_join_spec_change(self, entry):
        value = entry.get_text().strip()
        self.subject.joinSpec = value


@PropertyPages.register(UML.ControlFlow)
@PropertyPages.register(UML.ObjectFlow)
class FlowPropertyPageAbstract(PropertyPageBase):
    """Flow item element editor."""

    order = 15

    subject: UML.ActivityEdge

    def __init__(self, subject):
        self.subject = subject
        self.watcher = subject and subject.watcher()

    def construct(self):
        subject = self.subject

        if not subject:
            return

        builder = new_builder(
            "transition-editor",
            signals={
                "transition-destroy": (self.watcher.unsubscribe_all,),
            },
        )

        guard = builder.get_object("guard")
        guard.set_text(subject.guard or "")

        @handler_blocking(guard, "changed", self._on_guard_change)
        def handler(event):
            v = event.new_value
            if v != guard.get_text():
                guard.set_text(v or "")

        self.watcher.watch("guard", handler)

        return builder.get_object("transition-editor")

    @transactional
    def _on_guard_change(self, entry):
        value = entry.get_text().strip()
        self.subject.guard = value



from typing import Dict, Iterable

from gaphor.abc import ActionProvider, ModelingLanguage, Service
from gaphor.action import action
from gaphor.core import event_handler
from gaphor.entrypoint import initialize
from gaphor.services.properties import PropertyChanged


class ModelingLanguageChanged:
    def __init__(self, modeling_language):
        self.modeling_language = modeling_language


class ModelingLanguageService(Service, ActionProvider, ModelingLanguage):

    DEFAULT_LANGUAGE = "UML"

    def __init__(self, event_manager=None, properties=None):
        """Create a new Model Provider. It will provide all models defined as
        entrypoints under `[gaphor.modelinglanguages]`.

        The `properties` argument is optional, in which case the service
        will default to UML.
        """
        if properties is None:
            properties = {}
        self.event_manager = event_manager
        self.properties = properties

        self._modeling_languages: Dict[str, ModelingLanguage] = initialize(
            "gaphor.modelinglanguages"
        )
        if event_manager:
            self.event_manager.subscribe(self.on_property_changed)

    def shutdown(self):
        if self.event_manager:
            self.event_manager.unsubscribe(self.on_property_changed)

    @property
    def modeling_languages(self) -> Iterable[tuple[str, str]]:
        """An Iterator, returns tuples (id, localized name)."""
        for id, provider in self._modeling_languages.items():
            if provider.name:
                yield id, provider.name

    @property
    def active_modeling_language(self) -> str:
        """The identifier for the currently active modeling language."""
        modeling_language: str = self.properties.get(
            "modeling-language", self.DEFAULT_LANGUAGE
        )
        if modeling_language not in self._modeling_languages:
            modeling_language = self.DEFAULT_LANGUAGE
        return modeling_language

    @property
    def name(self) -> str:
        """Localized name of the active modeling language."""
        return self._modeling_language().name

    @property
    def toolbox_definition(self):
        return self._modeling_language().toolbox_definition

    @property
    def diagram_types(self):
        return self._modeling_language().diagram_types

    def lookup_element(self, name):
        return next(
            filter(
                None,
                map(
                    lambda provider: provider.lookup_element(name),
                    self._modeling_languages.values(),
                ),
            ),
            None,
        )

    @action(name="select-modeling-language")
    def select_modeling_language(self, modeling_language: str):
        self.properties.set("modeling-language", modeling_language)
        if self.event_manager:
            self.event_manager.handle(ModelingLanguageChanged(modeling_language))

    def _modeling_language(self) -> ModelingLanguage:
        return self._modeling_languages[self.active_modeling_language]

    @event_handler(PropertyChanged)
    def on_property_changed(self, event: PropertyChanged):
        if event.key == "modeling-language" and self.event_manager:
            self.event_manager.handle(ModelingLanguageChanged(event.new_value))



"""A generic way to handle errors in GUI applications.

This module also contains a ErrorHandlerAspect, which can be easily
attached to a class method and will raise the error dialog when the
method exits with an exception.
"""

import pdb
import sys

from gi.repository import Gtk

if Gtk.get_major_version() == 4:
    from gi.repository import Adw

from gaphor.i18n import gettext


def error_handler(message, secondary_message="", window=None, close=None):
    _exc_type, _exc_value, exc_traceback = sys.exc_info()

    debug_body = (f"{secondary_message}\n\n" if secondary_message else "") + gettext(
        "It looks like Gaphor is started from the command line. Do you want to open a debug session?"
    )
    if Gtk.get_major_version() == 3:
        dialog = Gtk.MessageDialog(message_type=Gtk.MessageType.ERROR, text=message)
        dialog.set_transient_for(window)

        if __debug__ and exc_traceback and sys.stdin.isatty():
            dialog.props.secondary_text = debug_body
            dialog.add_buttons(gettext("Close"), 0, gettext("Start Debug Session"), 100)
        else:
            dialog.props.secondary_text = secondary_message
            dialog.add_button(gettext("Close"), 0)
        dialog.set_modal(True)
    else:
        dialog = Adw.MessageDialog.new(
            window,
            message,
        )
        if __debug__ and exc_traceback and sys.stdin.isatty():
            dialog.set_body(debug_body)
            dialog.add_response("close", gettext("Close"))
            dialog.add_response("debug", gettext("Start Debug Session"))
            dialog.set_default_response("debug")
        else:
            dialog.set_body(secondary_message)
            dialog.add_response("close", gettext("Close"))
            dialog.set_default_response("close")
        dialog.set_close_response("close")

    def response(dialog, answer):
        dialog.destroy()
        if exc_traceback and answer in [100, "debug"]:
            pdb.post_mortem(exc_traceback)
        elif close:
            close()

    dialog.connect("response", response)
    dialog.show()



"""Defines a status window class for displaying the progress of a queue."""


import contextlib
from queue import Empty

from gi.repository import Gdk, GLib, Gtk, Pango


class StatusWindow:
    """Create a borderless window on the parent, usually the main window, with
    a label and a progress bar.

    The progress bar is updated as the queue is updated.
    """

    def __init__(self, title, message, parent=None, queue=None, display=True):
        """Create the status window.

        The title parameter is the title of the window.  The message
        parameter is a string displayed near the progress bar to
        indicate what is happening.  The parent parameter is the parent
        window to display the window in.  The queue parameter is a queue
        that is used to update the progress bar.  The display parameter
        will display the window if true.  This is the default.
        """

        self.title = title
        self.message = message
        self.parent = parent
        self.queue = queue
        self.window: Gtk.Window = None

        if display:
            self.display()

    def init_window(self):
        """Create the window GUI component.

        This will set the window and progress bar attributes, so they
        can be referenced later.
        """

        frame = Gtk.Frame.new(None)
        vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, spacing=12)
        label = Gtk.Label.new(self.message)
        label.set_ellipsize(Pango.EllipsizeMode.MIDDLE)

        self.progress_bar = Gtk.ProgressBar.new()
        self.progress_bar.set_size_request(400, -1)

        if Gtk.get_major_version() == 3:
            self.create_gtk3_window_with_progress_bar(frame, vbox, label)
        else:
            self.window = Gtk.Window.new()
            self.window.set_child(frame)
            frame.set_child(vbox)
            vbox.append(label)
            vbox.append(self.progress_bar)

        self.window.set_title(self.title)
        self.window.get_style_context().add_class("status-window")
        if self.parent:
            self.window.set_transient_for(self.parent)
        self.window.set_modal(True)
        self.window.set_resizable(False)
        self.window.set_decorated(False)

    def create_gtk3_window_with_progress_bar(self, frame, vbox, label):
        self.window = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        self.window.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
        self.window.set_keep_above(True)
        self.window.set_type_hint(Gdk.WindowTypeHint.SPLASHSCREEN)
        self.window.add(frame)
        frame.add(vbox)
        vbox.pack_start(label, True, True, 0)
        vbox.pack_start(self.progress_bar, expand=False, fill=False, padding=0)

    def display(self):
        """Display the status window.

        If a queue has been supplied to the StatusWindow instance, a new
        gobject idle handle is created.  Once the window is destroyed,
        this handler is removed.
        """
        if not self.window:
            self.init_window()

        assert self.window

        if Gtk.get_major_version() == 3:
            self.window.show_all()
        else:
            self.window.show()

        if self.queue:
            self.idle_id = GLib.idle_add(
                progress_idle_handler,
                self.progress_bar,
                self.queue,
                priority=GLib.PRIORITY_DEFAULT_IDLE,
            )
            self.window.connect("destroy", remove_idle_handler, self.idle_id)

    def destroy(self):
        """Destroy the status window.

        This will also remove the gobject handler.
        """
        if self.window:
            self.window.destroy()
            self.window = None


def progress_idle_handler(progress_bar, queue):
    """This is a gobject idle handler that updates the supplied progress bar.

    The percentage is retrieved from the queue until it is empty.  The
    progress bar is then updated with the current percentage.
    """
    percentage = 0
    with contextlib.suppress(Empty):
        percentage = queue.get(block=False)
    if percentage:
        progress_bar.set_fraction(min(percentage, 100.0) / 100.0)
    return GLib.SOURCE_CONTINUE


def remove_idle_handler(window, idle_id):
    """This removes the supplied gobject idle id.

    This handle is required by StatusWindow when it is destroyed.
    """

    GLib.source_remove(idle_id)



from __future__ import annotations

from functools import singledispatch

from gaphas import Item
from gaphas.geometry import Rectangle
from gi.repository import Gdk, Gtk

from gaphor.diagram.presentation import LinePresentation, Named
from gaphor.transaction import Transaction


@singledispatch
def instant_editor(
    item: Item, view, event_manager, pos: tuple[int, int] | None = None
) -> bool:
    """Show a small editor popup in the diagram. Makes for easy editing without
    resorting to the Element editor.

    In case of a mouse press event, the mouse position (relative to the
    element) are also provided.
    """
    return False


@instant_editor.register(Named)
def named_item_editor(item, view, event_manager, pos=None) -> bool:
    """Text edit support for Named items."""

    subject = item.subject
    if not subject:
        return False

    if isinstance(item, LinePresentation):
        box = item.middle_shape_size
        i2v = view.get_matrix_i2v(item)
        x, y = i2v.transform_point(box.x, box.y)
        w, h = i2v.transform_distance(box.width, box.height)
        box = Rectangle(x, y, w, h)
    else:
        box = view.get_item_bounding_box(item)
    name = subject.name or ""
    entry = popup_entry(name)

    def update_text():
        with Transaction(event_manager):
            item.subject.name = entry.get_buffer().get_text()

    show_popover(entry, view, box, update_text)

    return True


def popup_entry(text, update_text=None, done=None):
    buffer = Gtk.EntryBuffer()
    buffer.set_text(text, -1)
    entry = Gtk.Entry.new_with_buffer(buffer)
    entry.show()
    return entry


def show_popover(widget, view, box, commit):
    popover = Gtk.Popover.new()
    if Gtk.get_major_version() == 3:
        popover.add(widget)
        popover.set_relative_to(view)
    else:
        popover.set_child(widget)
        popover.set_parent(view)

    gdk_rect = Gdk.Rectangle()
    gdk_rect.x = box.x
    gdk_rect.y = box.y
    gdk_rect.width = box.width
    gdk_rect.height = box.height
    popover.set_pointing_to(gdk_rect)
    popover.set_position(Gtk.PositionType.TOP)

    should_commit = True

    def on_closed(popover):
        if should_commit:
            commit()
        view.grab_focus()

    popover.connect("closed", on_closed)

    def on_escape(popover, keyval, keycode, state):
        if keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter) and not state & (
            Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK
        ):
            popover.popdown()
            return True
        elif keyval == Gdk.KEY_Escape:
            nonlocal should_commit
            should_commit = False

    if Gtk.get_major_version() == 3:

        def on_escape3(popover, event):
            return on_escape(popover, event.keyval, 0, event.get_state())

        popover.connect("key-press-event", on_escape3)
        popover.popup()
    else:
        controller = Gtk.EventControllerKey.new()
        popover.add_controller(controller)
        controller.connect("key-pressed", on_escape)

        if isinstance(widget, Gtk.Entry):
            widget.connect("activate", lambda w: popover.popdown())

        if popover.get_root():
            # Test for root window to avoid segfaults in unit test
            popover.show()

    return popover



import logging

from gi.repository import Gdk, Gtk

from gaphor import UML
from gaphor.core import gettext, transactional
from gaphor.core.format import format, parse
from gaphor.diagram.hoversupport import widget_add_hover_support
from gaphor.diagram.propertypages import (
    EditableTreeModel,
    PropertyPageBase,
    PropertyPages,
    handler_blocking,
    help_link,
    new_resource_builder,
    on_bool_cell_edited,
    on_text_cell_edited,
)
from gaphor.UML.classes.datatype import DataTypeItem
from gaphor.UML.classes.interface import Folded, InterfaceItem
from gaphor.UML.classes.klass import ClassItem
from gaphor.UML.deployments.connector import ConnectorItem

log = logging.getLogger(__name__)


new_builder = new_resource_builder("gaphor.UML.classes")


@transactional
def on_keypress_event(ctrl, keyval, keycode, state, tree):
    k = Gdk.keyval_name(keyval).lower()
    if state & Gdk.ModifierType.CONTROL_MASK:
        return False

    if k in ("backspace", "delete"):
        model, iter = tree.get_selection().get_selected()
        if iter:
            model.remove(iter)
    elif k in ("equal", "plus"):
        model, iter = tree.get_selection().get_selected()
        model.swap(iter, model.iter_next(iter))
        return True
    elif k in ("minus", "underscore"):
        model, iter = tree.get_selection().get_selected()
        model.swap(iter, model.iter_previous(iter))
        return True
    return False


class ClassAttributes(EditableTreeModel):
    """GTK tree model to edit class attributes."""

    def __init__(self, item):
        super().__init__(item, cols=(str, bool, object))

    def get_rows(self):
        for attr in self._item.subject.ownedAttribute:
            if not attr.association:
                yield [format(attr, note=True), attr.isStatic, attr]

    def create_object(self):
        attr = self._item.model.create(UML.Property)
        self._item.subject.ownedAttribute = attr
        return attr

    @transactional
    def set_object_value(self, row, col, value):
        attr = row[-1]
        if col == 0:
            parse(attr, value)
            row[0] = format(attr, note=True)
        elif col == 1:
            attr.isStatic = not attr.isStatic
            row[1] = attr.isStatic
        elif col == 2:
            # Value in attribute object changed:
            row[0] = format(attr, note=True)
            row[1] = attr.isStatic

    def swap_objects(self, o1, o2):
        return self._item.subject.ownedAttribute.swap(o1, o2)

    def sync_model(self, new_order):
        self._item.subject.ownedAttribute.order(
            lambda e: new_order.index(e) if e in new_order else len(new_order)
        )


class ClassOperations(EditableTreeModel):
    """GTK tree model to edit class operations."""

    def __init__(self, item):
        super().__init__(item, cols=(str, bool, bool, object))

    def get_rows(self):
        for operation in self._item.subject.ownedOperation:
            yield [
                format(operation, note=True),
                operation.isAbstract,
                operation.isStatic,
                operation,
            ]

    def create_object(self):
        operation = self._item.model.create(UML.Operation)
        self._item.subject.ownedOperation = operation
        return operation

    @transactional
    def set_object_value(self, row, col, value):
        operation = row[-1]
        if col == 0:
            parse(operation, value)
            row[0] = format(operation, note=True)
        elif col == 1:
            operation.isAbstract = not operation.isAbstract
            row[1] = operation.isAbstract
        elif col == 2:
            operation.isStatic = not operation.isStatic
            row[2] = operation.isStatic
        elif col == 3:
            row[0] = format(operation, note=True)
            row[1] = operation.isAbstract
            row[2] = operation.isStatic

    def swap_objects(self, o1, o2):
        return self._item.subject.ownedOperation.swap(o1, o2)

    def sync_model(self, new_order):
        self._item.subject.ownedOperation.order(new_order.index)


class ClassEnumerationLiterals(EditableTreeModel):
    """GTK tree model to edit enumeration literals."""

    def __init__(self, item):
        super().__init__(item, cols=(str, object))

    def get_rows(self):
        for literal in self._item.subject.ownedLiteral:
            yield [format(literal), literal]

    def create_object(self):
        literal = self._item.model.create(UML.EnumerationLiteral)
        self._item.subject.ownedLiteral = literal
        literal.enumeration = self._item.subject
        return literal

    @transactional
    def set_object_value(self, row, col, value):
        literal = row[-1]
        if col == 0:
            parse(literal, value)
            row[0] = format(literal)
        elif col == 1:
            # Value in attribute object changed:
            row[0] = format(literal)

    def swap_objects(self, o1, o2):
        return self._item.subject.ownedLiteral.swap(o1, o2)

    def sync_model(self, new_order):
        self._item.subject.ownedLiteral.order(new_order.index)


def tree_view_column_tooltips(tree_view, tooltips):
    assert tree_view.get_n_columns() == len(tooltips)

    def on_query_tooltip(widget, x, y, keyboard_mode, tooltip):
        if path_and_more := widget.get_path_at_pos(x, y):
            path, column, cx, cy = path_and_more
            n = widget.get_columns().index(column)
            if tooltips[n]:
                tooltip.set_text(tooltips[n])
                return True
        return False

    tree_view.connect("query-tooltip", on_query_tooltip)


@PropertyPages.register(UML.NamedElement)
class NamedElementPropertyPage(PropertyPageBase):
    """An adapter which works for any named item view.

    It also sets up a table view which can be extended.
    """

    order = 10

    def __init__(self, subject: UML.NamedElement):
        super().__init__()
        assert subject is None or isinstance(subject, UML.NamedElement), "%s" % type(
            subject
        )
        self.subject = subject
        self.watcher = subject.watcher() if subject else None

    def construct(self):
        if (
            not self.subject
            or UML.recipes.is_metaclass(self.subject)
            or isinstance(self.subject, UML.ActivityPartition)
        ):
            return

        assert self.watcher
        builder = new_builder(
            "named-element-editor",
            signals={
                "name-entry-destroyed": (self.watcher.unsubscribe_all,),
            },
        )

        subject = self.subject

        entry = builder.get_object("name-entry")
        entry.set_text(subject and subject.name or "")

        @handler_blocking(entry, "changed", self._on_name_changed)
        def handler(event):
            if event.element is subject and event.new_value != entry.get_text():
                entry.set_text(event.new_value or "")

        self.watcher.watch("name", handler)

        return builder.get_object("named-element-editor")

    @transactional
    def _on_name_changed(self, entry):
        if self.subject.name != entry.get_text():
            self.subject.name = entry.get_text()


@PropertyPages.register(UML.Classifier)
class ClassifierPropertyPage(PropertyPageBase):

    order = 15

    def __init__(self, subject):
        self.subject = subject

    def construct(self):
        if UML.recipes.is_metaclass(self.subject):
            return

        builder = new_builder(
            "classifier-editor",
            signals={"abstract-changed": (self._on_abstract_change,)},
        )

        abstract = builder.get_object("abstract")
        abstract.set_active(self.subject.isAbstract)

        return builder.get_object("classifier-editor")

    @transactional
    def _on_abstract_change(self, button, gparam):
        self.subject.isAbstract = button.get_active()


@PropertyPages.register(InterfaceItem)
class InterfacePropertyPage(PropertyPageBase):
    """Adapter which shows a property page for an interface view."""

    order = 15

    def __init__(self, item):
        self.item = item

    def construct(self):
        builder = new_builder(
            "interface-editor", signals={"folded-changed": (self._on_fold_change,)}
        )

        item = self.item

        connected_items = [
            c.item for c in item.diagram.connections.get_connections(connected=item)
        ]
        disallowed = (ConnectorItem,)
        can_fold = not any(map(lambda i: isinstance(i, disallowed), connected_items))

        folded = builder.get_object("folded")
        folded.set_active(item.folded != Folded.NONE)
        folded.set_sensitive(can_fold)

        return builder.get_object("interface-editor")

    @transactional
    def _on_fold_change(self, button, gparam):
        item = self.item

        fold = button.get_active()

        item.folded = Folded.PROVIDED if fold else Folded.NONE


@PropertyPages.register(DataTypeItem)
@PropertyPages.register(ClassItem)
@PropertyPages.register(InterfaceItem)
class AttributesPage(PropertyPageBase):
    """An editor for attributes associated with classes and interfaces."""

    order = 20

    def __init__(self, item):
        super().__init__()
        self.item = item
        self.watcher = item.subject and item.subject.watcher()

    def construct(self):
        if not self.item.subject:
            return

        self.model = ClassAttributes(self.item)

        builder = new_builder(
            "attributes-editor",
            "attributes-info",
            "static-label",
            signals={
                "show-attributes-changed": (self._on_show_attributes_change,),
                "attributes-name-edited": (on_text_cell_edited, self.model, 0),
                "attributes-static-edited": (on_bool_cell_edited, self.model, 1),
                "tree-view-destroy": (self.watcher.unsubscribe_all,),
                "attributes-info-clicked": (self.on_attributes_info_clicked),
            },
        )
        self.info = builder.get_object("attributes-info")
        if Gtk.get_major_version() == 3:
            widget_add_hover_support(builder.get_object("attributes-info-icon"))
        else:
            help_link(builder, "attributes-info-icon", "attributes-info")

        show_attributes = builder.get_object("show-attributes")
        show_attributes.set_active(self.item.show_attributes)

        tree_view: Gtk.TreeView = builder.get_object("attributes-list")
        tree_view.set_model(self.model)
        tree_view_column_tooltips(tree_view, ["", gettext("Static")])
        if Gtk.get_major_version() == 3:
            controller = self.key_controller = Gtk.EventControllerKey.new(tree_view)
        else:
            controller = Gtk.EventControllerKey.new()
            tree_view.add_controller(controller)
        controller.connect("key-pressed", on_keypress_event, tree_view)

        def handler(event):
            attribute = event.element
            for row in self.model:
                if row[-1] is attribute:
                    row[:] = [
                        format(attribute, note=True),
                        attribute.isStatic,
                        attribute,
                    ]

        self.watcher.watch("ownedAttribute.name", handler).watch(
            "ownedAttribute.isDerived", handler
        ).watch("ownedAttribute.visibility", handler).watch(
            "ownedAttribute.isStatic", handler
        ).watch(
            "ownedAttribute.lowerValue", handler
        ).watch(
            "ownedAttribute.upperValue", handler
        ).watch(
            "ownedAttribute.defaultValue", handler
        ).watch(
            "ownedAttribute.typeValue", handler
        )

        return builder.get_object("attributes-editor")

    @transactional
    def _on_show_attributes_change(self, button, gparam):
        self.item.show_attributes = button.get_active()
        self.item.request_update()

    def on_attributes_info_clicked(self, image, event):
        self.info.show()


@PropertyPages.register(DataTypeItem)
@PropertyPages.register(ClassItem)
@PropertyPages.register(InterfaceItem)
class OperationsPage(PropertyPageBase):
    """An editor for operations associated with classes and interfaces."""

    order = 30

    def __init__(self, item):
        super().__init__()
        self.item = item
        self.watcher = item.subject and item.subject.watcher()

    def construct(self):
        if not self.item.subject:
            return

        self.model = ClassOperations(self.item)

        builder = new_builder(
            "operations-editor",
            "operations-info",
            "static-label",
            "abstract-label",
            signals={
                "show-operations-changed": (self._on_show_operations_change,),
                "operations-name-edited": (on_text_cell_edited, self.model, 0),
                "operations-abstract-edited": (on_bool_cell_edited, self.model, 1),
                "operations-static-edited": (on_bool_cell_edited, self.model, 2),
                "tree-view-destroy": (self.watcher.unsubscribe_all,),
                "operations-info-clicked": (self.on_operations_info_clicked),
            },
        )

        self.info = builder.get_object("operations-info")
        if Gtk.get_major_version() == 3:
            widget_add_hover_support(builder.get_object("operations-info-icon"))
        else:
            help_link(builder, "operations-info-icon", "operations-info")

        show_operations = builder.get_object("show-operations")
        show_operations.set_active(self.item.show_operations)

        tree_view: Gtk.TreeView = builder.get_object("operations-list")
        tree_view.set_model(self.model)
        tree_view_column_tooltips(
            tree_view, ["", gettext("Abstract"), gettext("Static")]
        )
        if Gtk.get_major_version() == 3:
            controller = self.key_controller = Gtk.EventControllerKey.new(tree_view)
        else:
            controller = Gtk.EventControllerKey.new()
            tree_view.add_controller(controller)
        controller.connect("key-pressed", on_keypress_event, tree_view)

        def handler(event):
            operation = event.element
            for row in self.model:
                if row[-1] is operation:
                    row[:] = [
                        format(operation, note=True),
                        operation.isAbstract,
                        operation.isStatic,
                        operation,
                    ]

        self.watcher.watch("ownedOperation.name", handler).watch(
            "ownedOperation.isAbstract", handler
        ).watch("ownedOperation.visibility", handler).watch(
            "ownedOperation.ownedParameter.lowerValue", handler
        ).watch(
            "ownedOperation.ownedParameter.upperValue", handler
        ).watch(
            "ownedOperation.ownedParameter.typeValue", handler
        ).watch(
            "ownedOperation.ownedParameter.defaultValue", handler
        )

        return builder.get_object("operations-editor")

    @transactional
    def _on_show_operations_change(self, button, gparam):
        self.item.show_operations = button.get_active()
        self.item.request_update()

    def on_operations_info_clicked(self, image, event):
        self.info.show()


@PropertyPages.register(UML.Component)
class ComponentPropertyPage(PropertyPageBase):

    order = 15

    subject: UML.Component

    def __init__(self, subject):
        self.subject = subject

    def construct(self):
        subject = self.subject

        if not subject:
            return

        builder = new_builder(
            "component-editor",
            signals={"indirectly-instantiated-changed": (self._on_ii_change,)},
        )

        ii = builder.get_object("indirectly-instantiated")
        ii.set_active(subject.isIndirectlyInstantiated)

        return builder.get_object("component-editor")

    @transactional
    def _on_ii_change(self, button, gparam):
        """Called when user clicks "Indirectly instantiated" check button."""
        if subject := self.subject:
            subject.isIndirectlyInstantiated = button.get_active()



from gaphas.guide import GuidedItemMoveMixin
from gaphas.move import ItemMove
from gaphas.move import Move as MoveAspect

from gaphor.diagram.group import group, ungroup
from gaphor.diagram.tools.dropzone import DropZoneMoveMixin, grow_parent
from gaphor.UML.states.pseudostates import PseudostateItem
from gaphor.UML.states.state import StateItem
from gaphor.UML.states.statemachine import StateMachineItem
from gaphor.UML.states.transition import TransitionItem


class RegionDropZoneMoveMixin(DropZoneMoveMixin):
    """This drop zone aspect has some specifics to drop a state in the right
    region on a state (machine)."""

    def stop_move(self, pos):
        view = self.view
        new_parent = view.selection.dropzone_item
        if (
            not isinstance(new_parent, (StateItem, StateMachineItem))
            or not new_parent.subject
            or not new_parent.subject.region
        ):
            return super().stop_move(pos)

        item = self.item
        old_parent = item.parent

        if not item.subject:
            GuidedItemMoveMixin.stop_move(self, pos)
            return

        item_pos = view.get_matrix_v2i(new_parent).transform_point(*pos)
        target_subject = new_parent.subject_at_point(item_pos)

        if target_subject is item.subject.container:
            GuidedItemMoveMixin.stop_move(self, pos)
            return

        if old_parent and ungroup(old_parent.subject, item.subject):
            item.change_parent(None)
            old_parent.request_update()

        item_pos = view.get_matrix_v2i(new_parent).transform_point(*pos)
        target_subject = new_parent.subject_at_point(item_pos)
        if group(target_subject, item.subject):
            grow_parent(new_parent, item)
            item.change_parent(new_parent)

        GuidedItemMoveMixin.stop_move(self, pos)


@MoveAspect.register(StateItem)
@MoveAspect.register(PseudostateItem)
@MoveAspect.register(TransitionItem)
class DropZoneMove(RegionDropZoneMoveMixin, GuidedItemMoveMixin, ItemMove):
    pass



"""The definition for the blocks section of the toolbox."""

from gaphas.item import SE

from gaphor import UML
from gaphor.core import gettext
from gaphor.diagram.diagramtoolbox import ToolDef, ToolSection, new_item_factory
from gaphor.SysML import diagramitems as sysml_items
from gaphor.SysML import sysml
from gaphor.UML import diagramitems as uml_items
from gaphor.UML.classes.classestoolbox import (
    composite_association_config,
    shared_association_config,
)
from gaphor.UML.toolboxconfig import namespace_config


def sysml_enumeration_config(new_item: uml_items.EnumerationItem, name=None):
    namespace_config(new_item, name)
    new_item.as_sysml_value_type = True


blocks = ToolSection(
    gettext("Blocks"),
    (
        ToolDef(
            "toolbox-block",
            gettext("Block"),
            "gaphor-block-symbolic",
            "<Shift>B",
            new_item_factory(
                sysml_items.BlockItem, sysml.Block, config_func=namespace_config
            ),
            handle_index=SE,
        ),
        ToolDef(
            "toolbox-package",
            gettext("Package"),
            "gaphor-package-symbolic",
            "p",
            new_item_factory(
                uml_items.PackageItem,
                UML.Package,
                config_func=namespace_config,
            ),
            handle_index=SE,
        ),
        ToolDef(
            "toolbox-composite-association",
            gettext("Composite Association"),
            "gaphor-composite-association-symbolic",
            "<Shift>Z",
            new_item_factory(
                uml_items.AssociationItem,
                config_func=composite_association_config,
            ),
        ),
        ToolDef(
            "toolbox-shared-association",
            gettext("Shared Association"),
            "gaphor-shared-association-symbolic",
            "<Shift>Q",
            new_item_factory(
                uml_items.AssociationItem,
                config_func=shared_association_config,
            ),
        ),
        ToolDef(
            "toolbox-association",
            gettext("Association"),
            "gaphor-association-symbolic",
            "<Shift>A",
            new_item_factory(uml_items.AssociationItem),
        ),
        ToolDef(
            "toolbox-generalization",
            gettext("Generalization"),
            "gaphor-generalization-symbolic",
            "<Shift>G",
            new_item_factory(uml_items.GeneralizationItem),
        ),
        ToolDef(
            "toolbox-value-type",
            gettext("ValueType"),
            "gaphor-value-type-symbolic",
            "<Shift>L",
            new_item_factory(
                uml_items.DataTypeItem,
                sysml.ValueType,
                config_func=namespace_config,
            ),
        ),
        ToolDef(
            "toolbox-enumeration",
            gettext("Enumeration"),
            "gaphor-enumeration-symbolic",
            "<Shift>W",
            new_item_factory(
                uml_items.EnumerationItem,
                UML.Enumeration,
                config_func=sysml_enumeration_config,
            ),
            handle_index=SE,
        ),
        ToolDef(
            "toolbox-primitive",
            gettext("Primitive"),
            "gaphor-primitive-symbolic",
            "<Shift>H",
            new_item_factory(
                uml_items.DataTypeItem,
                UML.PrimitiveType,
                config_func=namespace_config,
            ),
            handle_index=SE,
        ),
    ),
)



from typing import NamedTuple

from gi.repository import Gio, GLib, Gtk

from gaphor.abc import ActionProvider


def apply_application_actions(component_registry, gtk_app):
    scope = "app"
    for _name, provider in component_registry.all(ActionProvider):
        for attrname, act in iter_actions(provider, scope):
            a = create_gio_action(act, provider, attrname)
            gtk_app.add_action(a)
            if act.shortcut:
                gtk_app.set_accels_for_action(f"{scope}.{act.name}", [act.shortcut])
    return gtk_app


if Gtk.get_major_version() == 3:

    class ActionGroup(NamedTuple):
        actions: Gio.SimpleActionGroup
        shortcuts: Gtk.AccelGroup

    def window_action_group(component_registry) -> ActionGroup:
        scope = "win"
        action_group = Gio.SimpleActionGroup.new()
        accel_group = Gtk.AccelGroup.new()

        for _name, provider in component_registry.all(ActionProvider):
            create_action_group(
                provider, scope, action_group=action_group, accel_group=accel_group
            )

        return ActionGroup(actions=action_group, shortcuts=accel_group)

    def _accel_handler(scope, name):
        return (
            lambda agrp, win, key, mod: win.get_action_group(scope)
            .lookup_action(name)
            .activate()
        )

    def create_action_group(
        provider, scope, action_group=None, accel_group=None
    ) -> ActionGroup:
        if not action_group:
            action_group = Gio.SimpleActionGroup.new()
        if not accel_group:
            accel_group = Gtk.AccelGroup.new()

        for attrname, act in iter_actions(provider, scope):
            a = create_gio_action(act, provider, attrname)
            action_group.add_action(a)
            for shortcut in act.shortcuts:
                key, mod = Gtk.accelerator_parse(shortcut)
                accel_group.connect(
                    key, mod, Gtk.AccelFlags.VISIBLE, _accel_handler(scope, act.name)
                )
        return ActionGroup(actions=action_group, shortcuts=accel_group)

else:

    class ActionGroup(NamedTuple):  # type: ignore[no-redef]
        actions: Gio.SimpleActionGroup
        shortcuts: Gio.ListModel

    def window_action_group(component_registry) -> ActionGroup:
        action_group = Gio.SimpleActionGroup.new()
        store = Gio.ListStore.new(Gtk.Shortcut)
        scope = "win"
        for _name, provider in component_registry.all(ActionProvider):
            for attrname, act in iter_actions(provider, scope):
                a = create_gio_action(act, provider, attrname)
                action_group.add_action(a)
                for shortcut in act.shortcuts:
                    store.append(_new_shortcut(shortcut, act.detailed_name))
        return ActionGroup(actions=action_group, shortcuts=store)

    def create_action_group(provider, scope) -> ActionGroup:  # type: ignore[misc]
        action_group = Gio.SimpleActionGroup.new()
        store = Gio.ListStore.new(Gtk.Shortcut)
        for attrname, act in iter_actions(provider, scope):
            a = create_gio_action(act, provider, attrname)
            action_group.add_action(a)
            for shortcut in act.shortcuts:
                store.append(_new_shortcut(shortcut, act.detailed_name))
        return ActionGroup(actions=action_group, shortcuts=store)

    def _new_shortcut(shortcut, detailed_name):
        return Gtk.Shortcut.new(
            trigger=Gtk.ShortcutTrigger.parse_string(shortcut),
            action=Gtk.NamedAction.new(detailed_name),
        )


def create_gio_action(act, provider, attrname):
    if act.state is not None:
        state = act.state(provider) if callable(act.state) else act.state
        a = Gio.SimpleAction.new_stateful(
            act.name, as_variant_type(act.arg_type), to_variant(state)
        )
        a.connect("change-state", _action_activate, provider, attrname)
    else:
        a = Gio.SimpleAction.new(act.name, as_variant_type(act.arg_type))
        a.connect("activate", _action_activate, provider, attrname)
    return a


def iter_actions(provider, scope):
    provider_class = type(provider)
    for attrname in dir(provider_class):
        method = getattr(provider_class, attrname)
        act = getattr(method, "__action__", None)
        if act and act.scope == scope:
            yield (attrname, act)


_GVARIANT_TYPE_STR = GLib.VariantType.new("s")
_GVARIANT_TYPE_INT = GLib.VariantType.new("i")
_GVARIANT_TYPE_BOOL = GLib.VariantType.new("b")


def as_variant_type(t):
    if t is None:
        return None
    elif t is str:
        return _GVARIANT_TYPE_STR
    elif t is int:
        return _GVARIANT_TYPE_INT
    elif t is bool:
        return _GVARIANT_TYPE_BOOL
    else:
        raise ValueError(f"No GVariantType declared for Python type {t}")


def to_variant(v):
    if v is None:
        return None
    elif isinstance(v, str):
        return GLib.Variant.new_string(v)
    elif isinstance(v, bool):
        return GLib.Variant.new_boolean(v)
    elif isinstance(v, int):
        return GLib.Variant.new_int32(v)
    else:
        raise ValueError(
            f"No GVariant mapping declared for Python value {v} of type {type(v)}"
        )


def from_variant(v):
    if v is None:
        return None
    elif _GVARIANT_TYPE_STR.equal(v.get_type()):
        return v.get_string()
    elif _GVARIANT_TYPE_INT.equal(v.get_type()):
        return v.get_int32()
    elif _GVARIANT_TYPE_BOOL.equal(v.get_type()):
        return v.get_boolean()
    else:
        raise ValueError(f"No Python mapping declared for GVariant value {v}")


def _action_activate(action, param, obj, name):
    method = getattr(obj, name)
    if param is not None:
        method(from_variant(param))
        if action.get_state_type():
            action.set_state(param)
    else:
        method()



"""C4 Model Language entrypoint."""

from gaphor.abc import ModelingLanguage
from gaphor.core.modeling import coremodel


class CoreModelingLanguage(ModelingLanguage):
    @property
    def name(self) -> str:
        return ""

    @property
    def toolbox_definition(self):
        raise ValueError("No toolbox for the core model")

    @property
    def diagram_types(self):
        raise ValueError("No diagram types for the core model")

    def lookup_element(self, name):
        return getattr(coremodel, name, None)


class MockModelingLanguage(ModelingLanguage):
    """This class can be used to instantly combine modeling languages."""

    def __init__(self, *modeling_languages: ModelingLanguage):
        self._modeling_languages = modeling_languages

    @property
    def name(self) -> str:
        return "Mock"

    @property
    def toolbox_definition(self):
        raise ValueError("No toolbox for the mock model")

    @property
    def diagram_types(self):
        return ()

    def lookup_element(self, name):
        return next(
            filter(
                None,
                map(
                    lambda provider: provider.lookup_element(name),
                    self._modeling_languages,
                ),
            ),
            None,
        )



"""Use case extension relationship."""

from gaphor import UML
from gaphor.core import gettext
from gaphor.diagram.presentation import LinePresentation, Named
from gaphor.diagram.shapes import Box, Text, draw_arrow_head
from gaphor.diagram.support import represents
from gaphor.UML.recipes import stereotypes_str


@represents(UML.Extend, head=UML.Extend.extendedCase, tail=UML.Extend.extension)
class ExtendItem(Named, LinePresentation):
    """Use case extension relationship."""

    def __init__(self, diagram, id=None):
        super().__init__(
            diagram,
            id,
            shape_middle=Box(
                Text(text=lambda: stereotypes_str(self.subject, (gettext("extend"),))),
                Text(text=lambda: self.subject.name or ""),
            ),
            style={"dash-style": (7.0, 5.0)},
        )

        self._handles[0].pos = (30, 20)
        self._handles[1].pos = (0, 0)

        self.watch("subject.appliedStereotype.classifier.name").watch(
            "subject[NamedElement].name"
        )
        self.draw_head = draw_arrow_head



from gaphor.core import gettext
from gaphor.core.modeling.properties import attribute
from gaphor.diagram.presentation import (
    Classified,
    ElementPresentation,
    from_package_str,
)
from gaphor.diagram.shapes import (
    Box,
    JustifyContent,
    Text,
    TextAlign,
    draw_border,
    draw_top_separator,
)
from gaphor.diagram.support import represents
from gaphor.diagram.text import FontStyle, FontWeight
from gaphor.SysML.sysml import Block, ValueType
from gaphor.UML.classes.klass import attribute_watches
from gaphor.UML.classes.stereotype import stereotype_compartments, stereotype_watches
from gaphor.UML.recipes import stereotypes_str
from gaphor.UML.umlfmt import format_property


@represents(Block)
class BlockItem(Classified, ElementPresentation[Block]):
    def __init__(self, diagram, id=None):
        super().__init__(diagram, id)

        self.watch("show_stereotypes", self.update_shapes).watch(
            "show_parts", self.update_shapes
        ).watch("show_references", self.update_shapes).watch(
            "show_values", self.update_shapes
        ).watch(
            "subject[NamedElement].name"
        ).watch(
            "subject[NamedElement].namespace.name"
        ).watch(
            "subject[Classifier].isAbstract", self.update_shapes
        ).watch(
            "subject[Class].ownedAttribute.aggregation", self.update_shapes
        )
        attribute_watches(self, "Block")
        stereotype_watches(self)

    show_stereotypes: attribute[int] = attribute("show_stereotypes", int)

    show_parts: attribute[int] = attribute("show_parts", int, default=False)

    show_references: attribute[int] = attribute("show_references", int, default=False)

    show_values: attribute[int] = attribute("show_values", int, default=False)

    def additional_stereotypes(self):
        from gaphor.RAAML import raaml

        if isinstance(self.subject, raaml.Situation):
            return [gettext("Situation")]
        elif isinstance(self.subject, raaml.ControlStructure):
            return [gettext("ControlStructure")]
        elif isinstance(self.subject, Block):
            return [gettext("block")]
        else:
            return ()

    def update_shapes(self, event=None):
        self.shape = Box(
            Box(
                Text(
                    text=lambda: stereotypes_str(
                        self.subject, self.additional_stereotypes()
                    )
                ),
                Text(
                    text=lambda: self.subject.name or "",
                    width=lambda: self.width - 4,
                    style={
                        "font-weight": FontWeight.BOLD,
                        "font-style": FontStyle.ITALIC
                        if self.subject and self.subject.isAbstract
                        else FontStyle.NORMAL,
                    },
                ),
                Text(
                    text=lambda: from_package_str(self),
                    style={"font-size": "x-small"},
                ),
                style={
                    "padding": (12, 4, 12, 4),
                    "justify-content": JustifyContent.START,
                },
            ),
            *(
                self.show_parts
                and self.subject
                and [
                    self.block_compartment(
                        gettext("parts"),
                        lambda a: not a.association and a.aggregation == "composite",
                    )
                ]
                or []
            ),
            *(
                self.show_references
                and self.subject
                and [
                    self.block_compartment(
                        gettext("references"),
                        lambda a: not a.association and a.aggregation != "composite",
                    )
                ]
                or []
            ),
            *(
                self.show_values
                and self.subject
                and [
                    self.block_compartment(
                        gettext("values"),
                        lambda a: isinstance(a.type, ValueType)
                        and a.aggregation == "composite",
                    )
                ]
                or []
            ),
            *(self.show_stereotypes and stereotype_compartments(self.subject) or []),
            style={
                "justify-content": JustifyContent.START,
            },
            draw=draw_border,
        )

    def block_compartment(self, name, predicate):
        # We need to fix the attribute value, since the for loop changes it.
        def lazy_format(attribute):
            return lambda: format_property(attribute) or gettext("unnamed")

        return Box(
            Text(
                text=name,
                style={
                    "padding": (0, 0, 4, 0),
                    "font-size": "x-small",
                    "font-style": FontStyle.ITALIC,
                },
            ),
            *(
                Text(text=lazy_format(attribute), style={"text-align": TextAlign.LEFT})
                for attribute in self.subject.ownedAttribute
                if predicate(attribute)
            ),
            style={
                "padding": (4, 4, 4, 4),
                "min-height": 8,
                "justify-content": JustifyContent.START,
            },
            draw=draw_top_separator,
        )



from gi.repository import GLib

from gaphor.core import event_handler
from gaphor.event import Notification

DELAY = 5_000


class InAppNotifier:
    def __init__(self, builder):
        self.revealer = builder.get_object("notification-revealer")
        self.message_label = builder.get_object("notification-message")
        self.auto_hide_id = 0
        close = builder.get_object("notification-close")
        close.connect("clicked", self.close_notification)

    @event_handler(Notification)
    def handle(self, event: Notification):
        if not self.revealer.get_reveal_child():
            self.message_label.set_text(event.message)
            self.revealer.set_reveal_child(True)
            self.auto_hide_id = GLib.timeout_add(DELAY, self.close_notification)

    def close_notification(self, *args):
        self.revealer.set_reveal_child(False)
        if self.auto_hide_id:
            GLib.source_remove(self.auto_hide_id)
            self.auto_hide_id = 0



"""The SysML Modeling Language module is the entrypoint for SysML related
assets."""

from typing import Iterable

from gaphor.abc import ModelingLanguage
from gaphor.core import gettext
from gaphor.diagram.diagramtoolbox import DiagramType, ToolboxDefinition
from gaphor.SysML import diagramitems, sysml
from gaphor.SysML.toolbox import sysml_diagram_types, sysml_toolbox_actions


class SysMLModelingLanguage(ModelingLanguage):
    @property
    def name(self) -> str:
        return gettext("SysML")

    @property
    def toolbox_definition(self) -> ToolboxDefinition:
        return sysml_toolbox_actions

    @property
    def diagram_types(self) -> Iterable[DiagramType]:
        yield from sysml_diagram_types

    def lookup_element(self, name):
        return getattr(sysml, name, None) or getattr(diagramitems, name, None)



"""Compiler for CSS selectors.

Based on cssselect2.compiler, written by Simon Sapin and Guillaume
Ayoub.
"""

import re
from functools import singledispatch

from gaphor.core.styling import parser

# http://dev.w3.org/csswg/selectors/#whitespace
split_whitespace = re.compile("[^ \t\r\n\f]+").findall


def compile_selector_list(input):
    """Compile a (comma-separated) list of selectors.

    Based on cssselect2.compiler.compile_selector_list().

    Returns a list of compiled selectors.
    """
    return [
        (compile_node(selector), selector.specificity)
        for selector in parser.parse(input)
    ]


@singledispatch
def compile_node(selector):
    """Dynamic dispatch selector nodes.

    Default behavior is to deny (no match).
    """
    raise parser.SelectorError("Unknown selector", selector)


@compile_node.register
def compile_compound_selector(selector: parser.CompoundSelector):
    sub_expressions = [compile_node(sel) for sel in selector.simple_selectors]
    return lambda el: all(expr(el) for expr in sub_expressions)


@compile_node.register
def compile_name_selector(selector: parser.LocalNameSelector):
    return lambda el: el.name() == selector.lower_local_name


def ancestors(el):
    if p := el.parent():
        yield p
        yield from ancestors(p)


def descendants(el):
    for c in el.children():
        yield c
        yield from descendants(c)


@compile_node.register
def compile_combined_selector(selector: parser.CombinedSelector):
    left_inside = compile_node(selector.left)
    if selector.combinator == " ":

        def left(el):
            return any(left_inside(e) for e in ancestors(el))

    elif selector.combinator == ">":

        def left(el):
            p = el.parent()
            return p is not None and left_inside(p)

    else:
        raise parser.SelectorError("Unknown combinator", selector.combinator)

    right = compile_node(selector.right)
    return lambda el: right(el) and left(el)


@compile_node.register
def compile_attribute_selector(selector: parser.AttributeSelector):
    name = selector.lower_name
    operator = selector.operator
    value = selector.value and selector.value.lower()

    if operator is None:
        return lambda el: bool(el.attribute(name))
    elif operator == "=":
        return lambda el: el.attribute(name) == value
    elif operator == "~=":
        return lambda el: value in split_whitespace(el.attribute(name))
    elif operator == "^=":
        return lambda el: value and el.attribute(name).startswith(value)
    elif operator == "$=":
        return lambda el: value and el.attribute(name).endswith(value)
    elif operator == "*=":
        return lambda el: value and value in el.attribute(name)
    elif operator == "|=":

        def pipe_equal_matcher(el):
            v = el.attribute(name)
            return v == value or v and v.startswith(f"{value}-")

        return pipe_equal_matcher
    else:
        raise parser.SelectorError("Unknown attribute operator", operator)


@compile_node.register
def compile_pseudo_class_selector(selector: parser.PseudoClassSelector):
    name = selector.name
    if name == "empty":
        return lambda el: not next(el.children(), 0)
    elif name in ("root", "hover", "focus", "active", "drop", "disabled"):
        return lambda el: name in el.state()
    else:
        raise parser.SelectorError("Unknown pseudo-class", name)


@compile_node.register
def compile_functional_pseudo_class_selector(
    selector: parser.FunctionalPseudoClassSelector,
):
    name = selector.name
    if name not in ("has", "is", "not"):
        raise parser.SelectorError("Unknown pseudo-class", name)

    sub_selectors = compile_selector_list(selector.arguments)
    selector.specificity = max(spec for _, spec in sub_selectors)
    if name == "has":
        return lambda el: any(
            any(sel(c) for sel, _ in sub_selectors) for c in descendants(el)
        )
    elif name == "is":
        return lambda el: any(sel(el) for sel, _ in sub_selectors)
    elif name == "not":
        return lambda el: not any(sel(el) for sel, _ in sub_selectors)



"""Artifact item."""

from gaphor import UML
from gaphor.core.modeling.properties import attribute
from gaphor.core.styling import FontWeight, JustifyContent
from gaphor.diagram.presentation import Classified, ElementPresentation
from gaphor.diagram.shapes import Box, Text, cairo_state, draw_border
from gaphor.diagram.support import represents
from gaphor.UML.classes.stereotype import stereotype_compartments


@represents(UML.Artifact)
class ArtifactItem(Classified, ElementPresentation):
    def __init__(self, diagram, id=None):
        super().__init__(diagram, id)

        self.watch("show_stereotypes", self.update_shapes)
        self.watch("subject[NamedElement].name")
        self.watch("subject.appliedStereotype", self.update_shapes)
        self.watch("subject.appliedStereotype.classifier.name")
        self.watch("subject.appliedStereotype.slot", self.update_shapes)
        self.watch("subject.appliedStereotype.slot.definingFeature.name")
        self.watch("subject.appliedStereotype.slot.value", self.update_shapes)

    show_stereotypes: attribute[int] = attribute("show_stereotypes", int)

    def update_shapes(self, event=None):
        self.shape = Box(
            Box(
                Text(
                    text=lambda: UML.recipes.stereotypes_str(self.subject),
                ),
                Text(
                    text=lambda: self.subject.name or "",
                    style={"font-weight": FontWeight.BOLD},
                ),
                style={"padding": (4, 32, 4, 4)},
                draw=draw_artifact_icon,
            ),
            *(self.show_stereotypes and stereotype_compartments(self.subject) or []),
            style={
                "justify-content": JustifyContent.START
                if self.diagram and self.children
                else JustifyContent.CENTER,
            },
            draw=draw_border
        )


def draw_artifact_icon(box, context, bounding_box):
    with cairo_state(context.cairo) as cr:
        cr.set_line_width(1.0)
        ear = 5
        w = 15
        icon_margin_x = 6
        ix = bounding_box.width - icon_margin_x - w
        icon_margin_y = 12

        iy = icon_margin_y

        cr.move_to(ix + w - ear, iy)
        h = 20
        for x, y in (
            (ix + w - ear, iy + ear),
            (ix + w, iy + ear),
            (ix + w - ear, iy),
            (ix, iy),
            (ix, iy + h),
            (ix + w, iy + h),
            (ix + w, iy + ear),
        ):
            cr.line_to(x, y)
        stroke_color = context.style["color"]
        cr.set_source_rgba(*stroke_color)
        cr.stroke()
